#!/usr/bin/env python

import csv
from .Interfaces import hAMRonizedResultIterator
from hAMRonization.constants import GENE_PRESENCE

required_metadata = [
    "analysis_software_version",
    "reference_database_version",
    "input_file_name",
]


class DeepArgIterator(hAMRonizedResultIterator):
    def __init__(self, source, metadata):
        metadata["analysis_software_name"] = "deeparg"
        metadata["reference_database_name"] = "deeparg_db"
        metadata["genetic_variation_type"] = GENE_PRESENCE
        self.metadata = metadata

        self.field_mapping = {
            "#ARG": "gene_symbol",
            "query-start": "input_gene_start",
            "query-end": "input_gene_stop",
            # not really but most appropriate field
            "read_id": "input_sequence_id",
            "predicted_ARG-class": "drug_class",
            "best-hit": "gene_name",
            "probability": None,
            "identity": "sequence_identity",
            "alignment-length": None,
            "alignment-bitscore": None,
            "alignment-evalue": None,
            "counts": None,
            # gather from splitting besthit
            "_reference_accession": "reference_accession",
        }

        super().__init__(source, self.field_mapping, self.metadata)

    def parse(self, handle):
        """
        Read each and return it
        """
        # skip any manually specified fields for later
        reader = csv.DictReader(handle, delimiter="\t")
        for result in reader:
            result["_reference_accession"] = result["best-hit"].split("|")[0]
            yield self.hAMRonize(result, self.metadata)



#!/usr/bin/env python

import csv
import re
import math
from .Interfaces import hAMRonizedResultIterator
from hAMRonization.constants import (
    NUCLEOTIDE_VARIANT,
    AMINO_ACID_VARIANT,
    GENE_PRESENCE,
)

required_metadata = [
    "analysis_software_version",
    "reference_database_version",
    "input_file_name",
]


class RgiIterator(hAMRonizedResultIterator):
    def __init__(self, source, metadata):
        metadata["analysis_software_name"] = "rgi"
        metadata["reference_database_name"] = "CARD"
        metadata["genetic_variation_type"] = "Gene presence detected"
        self.metadata = metadata

        with open(source) as fh:
            header = next(fh)
            # i.e. RGI-bwt
            if "Resistomes & Variants: Observed in Genome(s)" in header.strip().split(
                "\t"
            ):
                self.field_mapping = {
                    "ARO Term": "gene_symbol",
                    "ARO Accession": "reference_accession",
                    "Reference Model Type": "genetic_variation_type",
                    "Reference DB": "reference_database_name",
                    "Alleles with Mapped Reads": None,
                    "Reference Allele(s) Identity "
                    "to CARD Reference Protein (%)": "sequence_identity",
                    "Resistomes & Variants: Observed in Genome(s)": None,
                    "Resistomes & Variants: Observed in Plasmid(s)": None,
                    "Resistomes & Variants: Observed Pathogen(s)": None,
                    "Completely Mapped Reads": None,
                    "Mapped Reads with Flanking Sequence": None,
                    "All Mapped Reads": None,
                    "Average Percent Coverage": "coverage_percentage",
                    "Average Length Coverage (bp)": "input_gene_length",
                    "Average MAPQ (Completely Mapped Reads)": None,
                    "Number of Mapped Baits": None,
                    "Number of Mapped Baits with Reads": None,
                    "Average Number of reads per Bait": None,
                    "Number of reads per Bait " "Coefficient of Variation (%)": None,
                    "Number of reads mapping to baits "
                    "and mapping to complete gene": None,
                    "Number of reads mapping to baits and "
                    "mapping to complete gene (%)": None,
                    "Mate Pair Linkage (# reads)": None,
                    "Reference Length": "reference_gene_length",
                    "AMR Gene Family": "gene_name",
                    "Drug Class": "drug_class",
                    "Resistance Mechanism": "resistance_mechanism",
                }
            else:
                # normal RGI mode
                self.field_mapping = {
                    "ORF_ID": None,
                    "Contig": "input_sequence_id",
                    "Start": "input_gene_start",
                    "Stop": "input_gene_stop",
                    "Orientation": "strand_orientation",
                    "Cut_Off": None,
                    "Pass_Bitscore": None,
                    "Best_Hit_Bitscore": None,
                    "Best_Hit_ARO": "gene_symbol",
                    "Best_Identities": "sequence_identity",
                    "ARO": "reference_accession",
                    "Model_type": "genetic_variation_type",
                    "SNPs_in_Best_Hit_ARO": None,
                    "_nucleotide_mutation": "nucleotide_mutation",
                    "_amino_acid_mutation": "amino_acid_mutation",
                    "Other_SNPs": None,
                    "Drug Class": "drug_class",
                    "Resistance Mechanism": "resistance_mechanism",
                    "AMR Gene Family": "gene_name",
                    "Predicted_DNA": None,
                    "Predicted_Protein": None,
                    "CARD_Protein_Sequence": None,
                    "Percentage Length of " "Reference Sequence": "coverage_percentage",
                    "ID": None,
                    "Model_ID": None,
                    "Nudged": None,
                    "Note": None,
                }
                # if RGI is run on ORFs then Contig should be None
                # and input_sequence_id should the ORF_ID i.e., reverse of
                # rgi run on contig input
                # this checks for that
                try:
                    first_result = next(fh)
                    first_result = first_result.strip().split('\t')
                    if first_result[1] == '':
                        self.field_mapping['ORF_ID'] = 'input_sequence_id'
                        self.field_mapping['Contig'] = None
                except StopIteration:
                    pass

        super().__init__(source, self.field_mapping, self.metadata)

    def parse(self, handle):
        """
        Read each and return it
        """
        # skip any manually specified fields for later
        reader = csv.DictReader(handle, delimiter="\t")
        for result in reader:
            # rgi-bwt mode doesn't support variant mutations
            if "Model_type" not in result:
                result["_nucleotide_mutation"] = None
                result["_amino_acid_mutation"] = None
                result["Reference Model Type"] = GENE_PRESENCE
            # normal RGI model
            else:
                result["_nucleotide_mutation"] = None
                result["_amino_acid_mutation"] = None

                if result["SNPs_in_Best_Hit_ARO"] == "n/a":
                    result["SNPs_in_Best_Hit_ARO"] = None

                hgvs_mutations = []
                if (
                    result["Model_type"] == "protein variant model"
                    or result["Model_type"] == "protein overexpression model"
                ):
                    result["Model_type"] = AMINO_ACID_VARIANT

                    if result["SNPs_in_Best_Hit_ARO"]:
                        for mutation in result["SNPs_in_Best_Hit_ARO"].split(","):
                            hgvs_mutations.append(f"p.{mutation}")
                        result["_amino_acid_mutation"] = ",".join(hgvs_mutations)

                elif result["Model_type"] == "rrna variant model":
                    result["Model_type"] = NUCLEOTIDE_VARIANT
                    if result["SNPs_in_Best_Hit_ARO"]:
                        for mutation in result["SNPs_in_Best_Hit_ARO"].split(","):
                            _, ref, pos, alt, _ = re.split(r"(\D+)(\d+)(\D+)", mutation)
                            hgvs_mutations.append(f"n.{pos}{ref}>{alt}")
                        result["_nucleotide_mutation"] = ",".join(hgvs_mutations)
                else:
                    result["Model_type"] = GENE_PRESENCE

            # round down average length of coverage so its comparable to other
            # target lengths
            if "Average Length Coverage (bp)" in result:
                result["Average Length Coverage (bp)"] = math.floor(
                    float(result["Average Length Coverage (bp)"])
                )
            yield self.hAMRonize(result, self.metadata)



#!/usr/bin/env python

from collections import OrderedDict
from .Interfaces import hAMRonizedResultIterator
from hAMRonization.constants import GENE_PRESENCE

required_metadata = [
    "analysis_software_version",
    "reference_database_version",
    "input_file_name",
]


class ResFamsIterator(hAMRonizedResultIterator):
    def __init__(self, source, metadata):
        metadata["analysis_software_name"] = "resfams"
        metadata["reference_database_name"] = "resfams_hmms"
        metadata["genetic_variation_type"] = GENE_PRESENCE
        self.metadata = metadata

        # needed as indexing into the positions
        self.field_mapping = OrderedDict(
            [
                ("target name", None),
                # this is blank in resfams output
                ("accession1", None),
                ("query name", "gene_name"),
                # extract from query name
                ("_gene_symbol", "gene_symbol"),
                ("accession2", "reference_accession"),
                ("E-value_full", None),
                ("score_full", None),
                ("bias_full", None),
                ("E-value_best_domain", None),
                ("score_best_domain", None),
                ("bias_best_domain", None),
                ("exp", None),
                ("reg", None),
                ("clu", None),
                ("ov", None),
                ("env", None),
                ("dom", None),
                ("rep", None),
                ("inc", None),
                ("description of target", None),
            ]
        )

        super().__init__(source, self.field_mapping, self.metadata)

    def parse(self, handle):
        """
        Read each and return it
        """
        # skip any manually specified fields for later
        report_fieldnames = [
            x for x in self.field_mapping.keys() if not x.startswith("_")
        ]
        for result in handle:
            if not result.startswith("#"):
                result = dict(zip(report_fieldnames, result.split()))
                result["_gene_symbol"] = result["query name"].split("_")[0]
                yield self.hAMRonize(result, self.metadata)



import pytest
import json
import os
import csv
from contextlib import contextmanager
import hAMRonization
from hAMRonization.summarize import summarize_reports


@contextmanager
def not_raises(exception, msg):
    try:
        yield
    except exception:
        raise pytest.fail(msg)


def test_empty_input_report():
    """
    Hamronize an empty report and determine whether valid json and tsv are created
    """
    metadata = {
        "analysis_software_version": "3.6.10",
        "reference_database_version": "2019-Jul-28",
        "input_file_name": "Empty no header",
    }
    parsed_report = hAMRonization.parse(
        "data/raw_outputs/amrfinderplus/empty_report_with_header.tsv",
        metadata,
        "amrfinderplus",
    )
    assert len([report for report in parsed_report]) == 0

    parsed_report.write(output_location="amrfinderplus_empty.tsv", output_format="tsv")

    with open("amrfinderplus_empty.tsv") as fh:
        assert len([line for line in fh]) == 0

    parsed_report.write(
        output_location="amrfinderplus_empty.json", output_format="json"
    )

    with open("amrfinderplus_empty.json") as fh:
        data = json.load(fh)
    assert len(data) == 0

    os.remove("amrfinderplus_empty.json")
    os.remove("amrfinderplus_empty.tsv")


def test_summarize_empty_reports():
    """
    A bit complex test that checks whether empty json and tsv reports are successfully
    summarized (and that summarize removes redundancy correctly)
    """
    metadata = {
        "analysis_software_version": "3.6.10",
        "reference_database_version": "2019-Jul-28",
        "input_file_name": "Empty report",
    }
    parsed_report = hAMRonization.parse(
        "data/raw_outputs/amrfinderplus/empty_report_with_header.tsv",
        metadata,
        "amrfinderplus",
    )
    parsed_report.write(
        output_location="amrfinderplus_empty_summarize.tsv", output_format="tsv"
    )

    parsed_report.write(
        output_location="amrfinderplus_empty_summarize.json", output_format="json"
    )

    rgi_metadata = {
        "analysis_software_version": "5.1.0",
        "reference_database_version": "2019-Jul-28",
        "input_file_name": "Non-empty report",
    }
    rgi_parsed_report = hAMRonization.parse(
        "data/raw_outputs/rgi/rgi.txt", metadata, "rgi"
    )
    rgi_parsed_report.write(output_location="rgi_report.tsv", output_format="tsv")

    rgi_parsed_report.write(output_location="rgi_report.json", output_format="json")

    reports_order_1 = [
        "rgi_report.tsv",
        "amrfinderplus_empty_summarize.json",
        "amrfinderplus_empty_summarize.tsv",
        "rgi_report.json",
    ]

    reports_order_2 = [
        "amrfinderplus_empty_summarize.json",
        "amrfinderplus_empty_summarize.tsv",
        "rgi_report.tsv",
        "rgi_report.json",
    ]

    reports_order_3 = [
        "amrfinderplus_empty_summarize.tsv",
        "rgi_report.tsv",
        "rgi_report.json",
        "amrfinderplus_empty_summarize.json",
    ]

    reports_order_4 = [
        "amrfinderplus_empty_summarize.tsv",
        "amrfinderplus_empty_summarize.json",
    ]

    reports_order_5 = [
        "amrfinderplus_empty_summarize.tsv",
        "amrfinderplus_empty_summarize.json",
    ]

    for result_num, order in [
        (10, reports_order_1),
        (10, reports_order_2),
        (10, reports_order_3),
        (0, reports_order_4),
        (0, reports_order_5),
    ]:
        summarize_reports(order, "tsv", "summarize.tsv")
        summarize_reports(order, "json", "summarize.json")
        # summarize_reports(order, 'interactive')

        with open("summarize.tsv") as fh:
            reader = csv.reader(fh, delimiter="\t")
            summarize_results = [row for row in reader]
            assert len(summarize_results) == result_num + 1
        os.remove("summarize.tsv")

        with open("summarize.json") as fh:
            assert len(json.load(fh)) == result_num
        os.remove("summarize.json")

    for report in reports_order_1:
        os.remove(report)



import pytest
from contextlib import contextmanager
import hAMRonization


@contextmanager
def not_raises(exception, msg):
    try:
        yield
    except exception:
        raise pytest.fail(msg)


def test_abricate():
    metadata = {
        "analysis_software_version": "0.9.8",
        "reference_database_version": "2019-Jul-28",
    }
    parsed_report = hAMRonization.parse(
        "data/dummy/abricate/report.tsv", metadata, "abricate"
    )

    for result in parsed_report:

        # assert mandatory fields
        assert result.input_file_name == "Dummy"
        assert result.gene_symbol == "oqxA"
        assert (
            result.gene_name
            == "multidrug efflux RND transporter periplasmic adaptor subunit OqxA"
        )
        assert result.reference_database_name == "ncbi"
        assert result.reference_database_version == "2019-Jul-28"
        assert result.reference_accession == "NG_048024.1"
        assert result.analysis_software_name == "abricate"
        assert result.analysis_software_version == "0.9.8"
        assert result.genetic_variation_type == "gene_presence_detected"

        # optional fields - present in dummy dataset
        assert result.sequence_identity == 99.58
        assert result.input_sequence_id == "NZ_LR792628.1"
        assert result.input_gene_start == 1333608
        assert result.input_gene_stop == 1334783
        assert result.strand_orientation == "-"
        assert result.coverage_percentage == 100
        assert result.drug_class == "PHENICOL;QUINOLONE"

        # missing data in report
        assert result.coverage_depth is None
        assert result.coverage_ratio is None
        assert result.reference_gene_length is None
        assert result.reference_protein_length is None
        assert result.input_gene_length is None
        assert result.input_protein_length is None
        assert result.antimicrobial_agent is None
        assert result.resistance_mechanism is None
        assert result.input_protein_start is None
        assert result.input_protein_stop is None
        assert result.reference_protein_start is None
        assert result.reference_protein_stop is None
        assert result.reference_gene_start is None
        assert result.reference_gene_stop is None


def test_amrfinderplus():
    metadata = {
        "analysis_software_version": "3.6.10",
        "reference_database_version": "2019-Jul-28",
        "input_file_name": "Dummy",
    }
    parsed_report = hAMRonization.parse(
        "data/dummy/amrfinderplus/report.tsv", metadata, "amrfinderplus"
    )

    for result in parsed_report:
        # assert mandatory fields
        assert result.input_file_name == "Dummy"
        assert result.gene_symbol == "oqxA"
        assert (
            result.gene_name
            == "multidrug efflux RND transporter periplasmic adaptor subunit OqxA"
        )
        assert result.reference_database_name == "NCBI Reference Gene Database"
        assert result.reference_database_version == "2019-Jul-28"
        assert result.reference_accession == "WP_002914189.1"
        assert result.analysis_software_name == "amrfinderplus"
        assert result.analysis_software_version == "3.6.10"
        assert result.genetic_variation_type == "gene_presence_detected"

        # optional fields - present in dummy dataset
        assert result.sequence_identity == 99.49
        assert result.input_sequence_id == "NZ_LR792628.1"
        assert result.input_gene_start == 1333611
        assert result.input_gene_stop == 1334783
        assert result.strand_orientation == "-"
        assert result.coverage_percentage == 100
        assert result.drug_class == "PHENICOL/QUINOLONE"
        assert result.antimicrobial_agent == "PHENICOL/QUINOLONE"
        assert result.reference_protein_length == 391
        assert result.input_protein_length == 391

        # missing data in report
        assert result.reference_gene_length == None
        assert result.input_gene_length == None
        assert result.coverage_depth is None
        assert result.coverage_ratio is None
        assert result.resistance_mechanism is None
        assert result.input_protein_start is None
        assert result.input_protein_stop is None
        assert result.reference_protein_start is None
        assert result.reference_protein_stop is None
        assert result.reference_gene_start is None
        assert result.reference_gene_stop is None


def test_amrplusplus():
    metadata = {
        "analysis_software_version": "0.0.1",
        "reference_database_version": "2019-Jul-28",
        "input_file_name": "Dummy",
    }
    parsed_report = hAMRonization.parse(
        "data/dummy/amrplusplus/gene.tsv", metadata, "amrplusplus"
    )

    for result in parsed_report:
        # assert mandatory fields
        assert result.input_file_name == "Dummy"
        assert result.gene_symbol == "OQXA"
        assert result.gene_name == "Drug_and_biocide_RND_efflux_pumps"
        assert result.reference_database_name == "megares"
        assert result.reference_database_version == "2019-Jul-28"
        assert result.reference_accession == "MEG_4334"
        assert result.analysis_software_name == "amrplusplus"
        assert result.analysis_software_version == "0.0.1"
        assert result.genetic_variation_type == "gene_presence_detected"

        # optional fields - present in dummy dataset
        assert result.coverage_percentage == 96.7687
        assert result.drug_class == "Drug_and_biocide_resistance"

        # missing data in report
        assert result.sequence_identity is None
        assert result.input_sequence_id is None
        assert result.input_gene_start is None
        assert result.input_gene_stop is None
        assert result.strand_orientation is None
        assert result.reference_gene_length is None
        assert result.input_gene_length is None
        assert result.antimicrobial_agent is None
        assert result.reference_protein_length is None
        assert result.coverage_depth is None
        assert result.coverage_ratio is None
        assert result.input_protein_length is None
        assert result.resistance_mechanism is None
        assert result.input_protein_start is None
        assert result.input_protein_stop is None
        assert result.reference_protein_start is None
        assert result.reference_protein_stop is None
        assert result.reference_gene_start is None
        assert result.reference_gene_stop is None


def test_ariba():
    metadata = {
        "analysis_software_version": "0.0.1",
        "reference_database_version": "2019-Jul-28",
        "input_file_name": "Dummy",
        "reference_database_name": "ncbi",
    }
    parsed_report = hAMRonization.parse(
        "data/dummy/ariba/report.tsv", metadata, "ariba"
    )

    for result in parsed_report:
        # assert mandatory fields
        assert result.input_file_name == "Dummy"
        assert result.gene_symbol == "oqxA"
        assert result.gene_name == "oqxA.3003922.EU370913.1.46651_47827.5460"
        assert result.reference_database_name == "ncbi"
        assert result.reference_database_version == "2019-Jul-28"
        assert result.reference_accession == "oqxA.3003922.EU370913.1.46651_47827.5460"
        assert result.analysis_software_name == "ariba"
        assert result.analysis_software_version == "0.0.1"
        assert result.genetic_variation_type == "gene_presence_detected"

        # optional fields - present in dummy dataset
        assert result.sequence_identity == 99.57
        assert result.input_sequence_id == "oqxA.l15.c17.ctg.1"
        assert result.reference_gene_length == 1176
        assert result.coverage_depth == 64.2

        # missing data in report
        assert result.input_gene_start is None
        assert result.input_gene_stop is None
        assert result.strand_orientation is None
        assert result.coverage_percentage is None
        assert result.drug_class is None
        assert result.input_gene_length is None
        assert result.antimicrobial_agent is None
        assert result.reference_protein_length is None
        assert result.coverage_ratio is None
        assert result.input_protein_length is None
        assert result.resistance_mechanism is None
        assert result.input_protein_start is None
        assert result.input_protein_stop is None
        assert result.reference_protein_start is None
        assert result.reference_protein_stop is None
        assert result.reference_gene_start is None
        assert result.reference_gene_stop is None


def test_ariba_var():
    metadata = {
        "analysis_software_version": "0.0.1",
        "reference_database_version": "2019-Jul-28",
        "input_file_name": "Dummy",
        "reference_database_name": "ncbi",
    }
    parsed_report = hAMRonization.parse(
        "data/dummy/ariba/report_var.tsv", metadata, "ariba"
    )

    for result in parsed_report:
        # assert mandatory fields
        assert result.input_file_name == "Dummy"
        assert result.gene_symbol == "rpoB"
        assert result.gene_name == "rpoB.3003288.BA000007.3.4990267_4994296.5493"
        assert result.reference_database_name == "ncbi"
        assert result.reference_database_version == "2019-Jul-28"
        assert (
            result.reference_accession == "rpoB.3003288.BA000007.3.4990267_4994296.5493"
        )
        assert result.analysis_software_name == "ariba"
        assert result.analysis_software_version == "0.0.1"

        # optional fields - present in dummy dataset
        assert result.sequence_identity == 93.44
        assert result.input_sequence_id == "rpoB.l15.c4.ctg.2"
        assert result.reference_gene_length == 4029
        assert result.coverage_depth == 33.7
        assert result.amino_acid_mutation == "p.R529C"
        assert result.nucleotide_mutation == "n.877CGT>TGC"
        assert result.genetic_variation_type == "protein_variant_detected"


def test_kmerresistance():
    metadata = {
        "analysis_software_version": "0.0.1",
        "reference_database_version": "2019-Jul-28",
        "input_file_name": "Dummy",
    }
    parsed_report = hAMRonization.parse(
        "data/dummy/kmerresistance/results.res", metadata, "kmerresistance"
    )

    for result in parsed_report:
        # assert mandatory fields
        assert result.input_file_name == "Dummy"
        assert result.gene_symbol == "oqxA"
        assert result.gene_name == "oqxA_1"
        assert result.reference_database_name == "resfinder"
        assert result.reference_database_version == "2019-Jul-28"
        assert result.reference_accession == "oqxA_1_EU370913"
        assert result.analysis_software_name == "kmerresistance"
        assert result.analysis_software_version == "0.0.1"
        assert result.genetic_variation_type == "gene_presence_detected"

        # optional fields - present in dummy dataset
        assert result.sequence_identity == 99.57
        assert result.reference_gene_length == 1176
        assert result.coverage_depth == 96.31
        assert result.coverage_percentage == 100

        # missing data in report
        assert result.input_sequence_id is None
        assert result.input_gene_start is None
        assert result.input_gene_stop is None
        assert result.strand_orientation is None
        assert result.drug_class is None
        assert result.input_gene_length is None
        assert result.antimicrobial_agent is None
        assert result.reference_protein_length is None
        assert result.coverage_ratio is None
        assert result.input_protein_length is None
        assert result.resistance_mechanism is None
        assert result.input_protein_start is None
        assert result.input_protein_stop is None
        assert result.reference_protein_start is None
        assert result.reference_protein_stop is None
        assert result.reference_gene_start is None
        assert result.reference_gene_stop is None


def test_resfinder():
    metadata = {
        "analysis_software_version": "4.1.0",
        "reference_database_version": "2021-02-01",
        "input_file_name": "Dummy",
    }
    parsed_report = hAMRonization.parse(
        "data/dummy/pointfinder/PointFinder_results.txt", metadata, "pointfinder"
    )

    for result in parsed_report:
        # assert mandatory fields
        assert result.input_file_name == "Dummy"
        assert result.gene_symbol == "gyrA"
        assert result.gene_name == "gyrA"
        assert result.reference_database_name == "pointfinder"
        assert result.reference_database_version == "2021-02-01"
        assert result.reference_accession == "gyrA p.G81D"
        assert result.analysis_software_name == "pointfinder"
        assert result.analysis_software_version == "4.1.0"
        assert result.genetic_variation_type == "protein_variant_detected"

        assert result.drug_class == "Ciprofloxacin,Nalidixic acid,Ciprofloxacin"
        assert result.nucleotide_mutation == "GGT -> GAT"
        assert result.amino_acid_mutation == "p.G81D"


def test_pointfinder():
    metadata = {
        "analysis_software_version": "4.1.0",
        "reference_database_version": "2019-Jul-28",
        "input_file_name": "Dummy",
    }
    parsed_report = hAMRonization.parse(
        "data/dummy/resfinder/ResFinder_results_tab.txt", metadata, "resfinder"
    )


def test_rgi_variants():
    metadata = {
        "analysis_software_version": "5.2.0",
        "reference_database_version": "3.2.4",
        "input_file_name": "Dummy",
    }
    parsed_report = hAMRonization.parse("data/dummy/rgi/rgi_var.txt", metadata, "rgi")

    for result in parsed_report:
        # assert mandatory fields
        assert result.input_file_name == "Dummy"
        assert (
            result.gene_symbol
            == "Escherichia coli gyrB conferring resistance to aminocoumarin"
        )
        assert result.gene_name == "aminocoumarin resistant gyrB"
        assert result.reference_database_name == "CARD"
        assert result.reference_database_version == "3.2.4"
        assert result.reference_accession == "3003303"
        assert result.analysis_software_name == "rgi"
        assert result.analysis_software_version == "5.2.0"
        assert result.genetic_variation_type == "protein_variant_detected"

        # optional fields - present in dummy dataset
        assert result.drug_class == "aminocoumarin antibiotic"
        assert result.sequence_identity == 99.88
        assert result.resistance_mechanism == "antibiotic target alteration"

def test_rgi_orf_mode():
    metadata = {
        "analysis_software_version": "6.0.0",
        "reference_database_version": "3.2.5",
        "input_file_name": "Dummy ORF",
    }
    parsed_report = hAMRonization.parse("data/dummy/rgi/rgi_orf.txt", metadata, "rgi")

    for result in parsed_report:
        # assert mandatory fields
        assert result.input_file_name == "Dummy ORF"
        assert result.gene_symbol == "NDM-5"
        assert (
            result.gene_name
            == "NDM beta-lactamase"
        )
        assert result.reference_database_name == "CARD"
        assert result.reference_database_version == "3.2.5"
        assert result.reference_accession == "3000467"
        assert result.analysis_software_name == "rgi"
        assert result.analysis_software_version == "6.0.0"
        assert result.genetic_variation_type == "gene_presence_detected"

        # optional fields - present in dummy dataset
        assert result.input_sequence_id == "gb|AEN03071.1|+|NDM-5 [Escherichia coli]"
        assert result.input_gene_start == ''
        assert result.input_gene_stop == ''
        assert result.strand_orientation == ''
        assert (
            result.drug_class
            == "carbapenem; cephalosporin; cephamycin; penam"
        )
        assert result.sequence_identity == 100
        assert result.coverage_percentage == 100
        assert result.resistance_mechanism == "antibiotic inactivation"


def test_rgi():
    metadata = {
        "analysis_software_version": "5.1.0",
        "reference_database_version": "2019-Jul-28",
        "input_file_name": "Dummy",
    }
    parsed_report = hAMRonization.parse("data/dummy/rgi/rgi.txt", metadata, "rgi")

    for result in parsed_report:
        # assert mandatory fields
        assert result.input_file_name == "Dummy"
        assert result.gene_symbol == "oqxA"
        assert (
            result.gene_name
            == "resistance-nodulation-cell division (RND) antibiotic efflux pump"
        )
        assert result.reference_database_name == "CARD"
        assert result.reference_database_version == "2019-Jul-28"
        assert result.reference_accession == "3003922"
        assert result.analysis_software_name == "rgi"
        assert result.analysis_software_version == "5.1.0"
        assert result.genetic_variation_type == "gene_presence_detected"

        # optional fields - present in dummy dataset
        assert result.input_sequence_id == "NZ_LR792628.1_1289"
        assert result.input_gene_start == 1333608
        assert result.input_gene_stop == 1334783
        assert result.strand_orientation == "-"
        assert (
            result.drug_class
            == "fluoroquinolone antibiotic; glycylcycline; tetracycline antibiotic; diaminopyrimidine antibiotic; nitrofuran antibiotic"
        )
        assert result.sequence_identity == 99.49
        assert result.coverage_percentage == 100
        assert result.resistance_mechanism == "antibiotic efflux"

        # missing data in report
        assert result.reference_gene_length is None
        assert result.coverage_depth is None
        assert result.input_gene_length is None
        assert result.antimicrobial_agent is None
        assert result.reference_protein_length is None
        assert result.coverage_ratio is None
        assert result.input_protein_length is None
        assert result.input_protein_start is None
        assert result.input_protein_stop is None
        assert result.reference_protein_start is None
        assert result.reference_protein_stop is None
        assert result.reference_gene_start is None
        assert result.reference_gene_stop is None


def test_srax():
    metadata = {
        "analysis_software_version": "5.1.0",
        "reference_database_version": "2019-Jul-28",
        "input_file_name": "Dummy",
        "reference_database_name": "resfinder",
    }
    parsed_report = hAMRonization.parse(
        "data/dummy/srax/sraX_detected_ARGs.tsv", metadata, "srax"
    )

    for result in parsed_report:
        # assert mandatory fields
        assert result.input_file_name == "Dummy"
        assert result.gene_symbol == "oqxA"
        assert (
            result.gene_name
            == "RND efflux pump conferring resistance to fluoroquinolone"
        )
        assert result.reference_database_name == "resfinder"
        assert result.reference_database_version == "2019-Jul-28"
        assert result.reference_accession == "NG_048024.1"
        assert result.analysis_software_name == "srax"
        assert result.analysis_software_version == "5.1.0"
        assert result.genetic_variation_type == "gene_presence_detected"

        # optional fields - present in dummy dataset
        assert result.drug_class == "Fluoroquinolone"
        assert result.sequence_identity == 99.7
        assert result.coverage_percentage == 100

        # missing data in report
        assert result.input_sequence_id is None
        assert result.input_gene_start is None
        assert result.input_gene_stop is None
        assert result.strand_orientation is None
        assert result.resistance_mechanism is None
        assert result.reference_gene_length is None
        assert result.coverage_depth is None
        assert result.input_gene_length is None
        assert result.antimicrobial_agent is None
        assert result.reference_protein_length is None
        assert result.coverage_ratio is None
        assert result.input_protein_length is None
        assert result.input_protein_start is None
        assert result.input_protein_stop is None
        assert result.reference_protein_start is None
        assert result.reference_protein_stop is None
        assert result.reference_gene_start is None
        assert result.reference_gene_stop is None


def test_groot():
    metadata = {
        "analysis_software_version": "0.0.1",
        "reference_database_version": "2019-Jul-28",
        "input_file_name": "Dummy",
        "reference_database_name": "argannot",
    }
    parsed_report = hAMRonization.parse(
        "data/dummy/groot/groot_report.tsv", metadata, "groot"
    )

    for result in parsed_report:
        # assert mandatory fields
        assert result.input_file_name == "Dummy"
        assert result.gene_symbol == "OqxA"
        assert result.gene_name == "OqxA.3003470.EU370913"
        assert result.reference_database_name == "argannot"
        assert result.reference_database_version == "2019-Jul-28"
        assert (
            result.reference_accession == "OqxA.3003470.EU370913.4407527-4408202.4553"
        )
        assert result.analysis_software_name == "groot"
        assert result.analysis_software_version == "0.0.1"
        assert result.genetic_variation_type == "gene_presence_detected"

        # optional fields - present in dummy dataset
        assert result.reference_gene_length == 1176
        assert result.coverage_depth == 266

        # missing data in report
        assert result.input_sequence_id is None
        assert result.input_gene_start is None
        assert result.input_gene_stop is None
        assert result.strand_orientation is None
        assert result.resistance_mechanism is None
        assert result.input_gene_length is None
        assert result.antimicrobial_agent is None
        assert result.reference_protein_length is None
        assert result.coverage_ratio is None
        assert result.input_protein_length is None
        assert result.input_protein_start is None
        assert result.input_protein_stop is None
        assert result.reference_protein_start is None
        assert result.reference_protein_stop is None
        assert result.reference_gene_start is None
        assert result.reference_gene_stop is None
        assert result.drug_class is None
        assert result.sequence_identity is None
        assert result.coverage_percentage is None


def test_deeparg():
    metadata = {
        "analysis_software_version": "0.0.1",
        "reference_database_version": "2019-Jul-28",
        "input_file_name": "Dummy",
    }
    parsed_report = hAMRonization.parse(
        "data/dummy/deepARG/output.mapping.ARG.", metadata, "deeparg"
    )

    for result in parsed_report:
        # assert mandatory fields
        assert result.input_file_name == "Dummy"
        assert result.gene_symbol == "OQXA"
        assert result.gene_name == "YP_001693237.1|FEATURES|oqxA|multidrug|oqxA"
        assert result.reference_database_name == "deeparg_db"
        assert result.reference_database_version == "2019-Jul-28"
        assert result.reference_accession == "YP_001693237.1"
        assert result.analysis_software_name == "deeparg"
        assert result.analysis_software_version == "0.0.1"
        assert result.genetic_variation_type == "gene_presence_detected"

        # optional fields - present in dummy dataset
        assert result.drug_class == "multidrug"
        assert result.sequence_identity == 94.6
        assert result.input_sequence_id == "SNL153:124:HLM5WBCXX:1:2207:7453:53826"
        assert result.input_gene_start == 49
        assert result.input_gene_stop == 84

        # missing data in report
        assert result.coverage_percentage is None
        assert result.strand_orientation is None
        assert result.reference_gene_length is None
        assert result.input_gene_length is None
        assert result.antimicrobial_agent is None
        assert result.reference_protein_length is None
        assert result.coverage_depth is None
        assert result.coverage_ratio is None
        assert result.input_protein_length is None
        assert result.resistance_mechanism is None
        assert result.input_protein_start is None
        assert result.input_protein_stop is None
        assert result.reference_protein_start is None
        assert result.reference_protein_stop is None
        assert result.reference_gene_start is None
        assert result.reference_gene_stop is None


def test_srst2():
    metadata = {
        "analysis_software_version": "0.0.1",
        "reference_database_version": "2019-Jul-28",
        "input_file_name": "Dummy",
    }
    parsed_report = hAMRonization.parse(
        "data/dummy/srst2/report.tsv", metadata, "srst2"
    )

    for result in parsed_report:
        # assert mandatory fields
        assert result.input_file_name == "Dummy"
        assert result.gene_symbol == "oqxA"
        assert result.gene_name == "oqxA"
        assert result.reference_database_name == "ResFinder"
        assert result.reference_database_version == "2019-Jul-28"
        assert result.reference_accession == "1995"
        assert result.analysis_software_name == "srst2"
        assert result.analysis_software_version == "0.0.1"
        assert result.genetic_variation_type == "gene_presence_detected"

        # optional fields - present in dummy dataset
        assert result.coverage_percentage == 100
        assert result.reference_gene_length == 660
        assert result.coverage_depth == 75.852

        # missing data in report
        assert result.drug_class is None
        assert result.sequence_identity is None
        assert result.input_sequence_id is None
        assert result.input_gene_start is None
        assert result.input_gene_stop is None
        assert result.strand_orientation is None
        assert result.input_gene_length is None
        assert result.antimicrobial_agent is None
        assert result.reference_protein_length is None
        assert result.coverage_ratio is None
        assert result.input_protein_length is None
        assert result.resistance_mechanism is None
        assert result.input_protein_start is None
        assert result.input_protein_stop is None
        assert result.reference_protein_start is None
        assert result.reference_protein_stop is None
        assert result.reference_gene_start is None
        assert result.reference_gene_stop is None


def test_csstar():
    metadata = {
        "analysis_software_version": "0.0.1",
        "reference_database_version": "2019-Jul-28",
        "input_file_name": "Dummy",
        "reference_database_name": "ResFinder",
    }
    parsed_report = hAMRonization.parse(
        "data/dummy/sstar/report.tsv", metadata, "csstar"
    )

    for result in parsed_report:
        # assert mandatory fields
        assert result.input_file_name == "Dummy"
        assert result.gene_symbol == "oqxA"
        assert result.gene_name == "oqxA"
        assert result.reference_database_name == "ResFinder"
        assert result.reference_database_version == "2019-Jul-28"
        assert result.reference_accession == "oqxA"
        assert result.analysis_software_name == "csstar"
        assert result.analysis_software_version == "0.0.1"
        assert result.genetic_variation_type == "gene_presence_detected"

        # optional fields - present in dummy dataset
        assert result.reference_gene_length == 1176
        assert result.input_gene_length == 1176
        assert result.sequence_identity == 99.575
        assert result.input_sequence_id == "NZ_LR792628.1"

        # missing data in report
        assert result.coverage_percentage is None
        assert result.coverage_depth is None
        assert result.drug_class is None
        assert result.input_gene_start is None
        assert result.input_gene_stop is None
        assert result.strand_orientation is None
        assert result.antimicrobial_agent is None
        assert result.reference_protein_length is None
        assert result.coverage_ratio is None
        assert result.input_protein_length is None
        assert result.resistance_mechanism is None
        assert result.input_protein_start is None
        assert result.input_protein_stop is None
        assert result.reference_protein_start is None
        assert result.reference_protein_stop is None
        assert result.reference_gene_start is None
        assert result.reference_gene_stop is None


def test_tbprofiler():
    metadata = {}
    parsed_report = hAMRonization.parse(
        "data/dummy/tbprofiler/tbprofiler.json", metadata, "tbprofiler"
    )

    for result in parsed_report:
        # assert mandatory fields
        assert result.input_file_name == "tbprofiler.json"
        assert result.gene_symbol == "rpoB"
        assert result.gene_name == "rpoB"
        assert result.reference_database_name == "tbdb"
        assert result.reference_database_version == "a800e0a"
        assert result.analysis_software_name == "tb-profiler"
        assert result.analysis_software_version == "3.0.8"
        assert result.genetic_variation_type == "protein_variant_detected"
        assert result.reference_accession == "CCP43410"

        # optional fields - present in dummy dataset
        assert result.drug_class == "rifampicin"
        assert result.nucleotide_mutation == "c.1349C>T"
        assert result.amino_acid_mutation == "p.Ser450Leu"

        # todo
        assert result.amino_acid_mutation_interpretation is None
        assert result.amino_acid_mutation_interpretation is None

        # missing data in report
        assert result.sequence_identity is None
        assert result.reference_gene_length is None
        assert result.input_gene_length is None
        assert result.input_sequence_id is None
        assert result.coverage_percentage is None
        assert result.coverage_depth is None
        assert result.input_gene_start is None
        assert result.input_gene_stop is None
        assert result.strand_orientation is None
        assert result.antimicrobial_agent is None
        assert result.reference_protein_length is None
        assert result.coverage_ratio is None
        assert result.input_protein_length is None
        assert result.resistance_mechanism is None
        assert result.input_protein_start is None
        assert result.input_protein_stop is None
        assert result.reference_protein_start is None
        assert result.reference_protein_stop is None
        assert result.reference_gene_start is None
        assert result.reference_gene_stop is None


def test_mykrobe():
    metadata = {}
    parsed_report = hAMRonization.parse(
        "data/dummy/mykrobe/mykrobe.json", metadata, "mykrobe"
    )

    for result in parsed_report:
        # assert mandatory fields
        assert result.input_file_name == "mykrobe.json"
        assert result.gene_symbol == "rpoB"
        assert result.gene_name == "rpoB"
        assert (
            result.reference_database_name
            == "tb/tb-species-170421.fasta.gz;tb/tb-hunt-probe-set-jan-03-2019.fasta.gz;tb/tb.lineage.20200930.probes.fa.gz"
        )
        assert result.reference_database_version == "v0.10.0"
        assert result.reference_accession == "NC_000962.3"
        assert result.analysis_software_name == "mykrobe"
        assert result.analysis_software_version == "v0.10.0"
        assert result.genetic_variation_type == "protein_variant_detected"

        # optional fields - present in dummy dataset
        assert (
            result.drug_class == "Rifampicin"
        )  # TODO: this is not following the spec as this is not an ARO term, what to do?
        assert result.coverage_percentage == 100
        assert result.coverage_depth == 60
        assert result.amino_acid_mutation == "p.Ser450Leu"
        # assert result.nucleotide_mutation == ""  # TODO: this is not working yet

        # missing data in report
        assert result.sequence_identity is None
        assert result.reference_gene_length is None
        assert result.input_gene_length is None
        assert result.input_sequence_id is None
        assert result.input_gene_start is None
        assert result.input_gene_stop is None
        assert result.strand_orientation is None
        assert result.antimicrobial_agent is None
        assert result.reference_protein_length is None
        assert result.coverage_ratio is None
        assert result.input_protein_length is None
        assert result.resistance_mechanism is None
        assert result.input_protein_start is None
        assert result.input_protein_stop is None
        assert result.reference_protein_start is None
        assert result.reference_protein_stop is None
        assert result.reference_gene_start is None
        assert result.reference_gene_stop is None


def test_staramr_resfinder():
    metadata = {
        "analysis_software_version": "0.0.1",
        "reference_database_version": "2019-Jul-28",
    }
    parsed_report = hAMRonization.parse(
        "data/dummy/staramr/resfinder.tsv", metadata, "staramr"
    )
    for result in parsed_report:
        # assert mandatory fields
        assert result.input_file_name == "GCF_902827215.1_SB5881_genomic"
        assert result.gene_symbol == "oqxA"
        assert result.gene_name == "oqxA"
        assert result.reference_database_name == "resfinder/pointfinder"
        assert result.reference_database_version == "2019-Jul-28"
        assert result.reference_accession == "EU370913"
        assert result.analysis_software_name == "staramr"
        assert result.analysis_software_version == "0.0.1"

        # optional fields - present in dummy dataset
        assert result.input_sequence_id == "ref|NZ_LR792628.1|"
        assert result.input_gene_start == 1334783
        assert result.input_gene_stop == 1333608
        assert result.drug_class == "chloramphenicol"
        assert result.sequence_identity == 99.58


def test_staramr_pointfinder():
    metadata = {
        "analysis_software_version": "0.0.1",
        "reference_database_version": "2019-Jul-28",
    }
    parsed_report = hAMRonization.parse(
        "data/dummy/staramr/pointfinder.tsv", metadata, "staramr"
    )
    for result in parsed_report:
        # assert mandatory fields
        assert result.input_file_name == "SRR1952908"
        assert result.gene_name == "gyrA (S83Y)"
        assert result.gene_symbol == "gyrA (S83Y)"
        assert result.reference_database_name == "resfinder/pointfinder"
        assert result.reference_database_version == "2019-Jul-28"
        assert result.reference_accession == "gyrA (S83Y)"
        assert result.analysis_software_name == "staramr"
        assert result.analysis_software_version == "0.0.1"

        # optional fields - present in dummy dataset
        assert result.input_gene_start == 22801
        assert result.input_gene_stop == 20165
        assert result.drug_class == "ciprofloxacin I/R, nalidixic acid"
        assert result.sequence_identity == 99.96
        assert result.coverage_percentage == 2367 / 2367 * 100
        assert result.nucleotide_mutation == "n.83TCC>TAC"
        assert result.amino_acid_mutation == "p.S83Y"


def test_resfams():
    metadata = {
        "analysis_software_version": "0.0.1",
        "reference_database_version": "2019-Jul-28",
        "input_file_name": "Dummy",
    }
    parsed_report = hAMRonization.parse(
        "data/dummy/resfams/resfams.tblout", metadata, "resfams"
    )
    for result in parsed_report:
        assert result.gene_name == "Acetyltransf_4"
        assert result.gene_symbol == "Acetyltransf"
        assert result.reference_accession == "RF0013"
        assert result.input_file_name == "Dummy"
        assert result.analysis_software_name == "resfams"
        assert result.analysis_software_version == "0.0.1"
        assert result.genetic_variation_type == "gene_presence_detected"
        assert result.reference_database_name == "resfams_hmms"

def test_fargene():
    metadata = {
            'analysis_software_version': '0.1',
            'input_file_name': 'Dummy fargene',
            'reference_database_version': '0.1'}

    parsed_report = hAMRonization.parse(
            'data/dummy/fargene/retrieved-genes-class_A-hmmsearched.out',
            metadata, 'fargene')

    for result in parsed_report:
        assert result.gene_name == 'classA_70_centroids-aligned'
        assert result.gene_symbol == 'classA'
        assert result.drug_class == 'classA'
        assert result.reference_accession == 'classA_70_centroids-aligned'

        assert result.input_file_name == 'Dummy fargene'
        assert result.analysis_software_name == 'fargene'
        assert result.analysis_software_version == '0.1'
        assert result.reference_database_version == '0.1'
        assert result.genetic_variation_type == 'gene_presence_detected'
        assert result.reference_database_name == 'fargene_hmms'
        assert result.input_sequence_id == 'contigs_NODE_6_length_23263_cov_4.891675_seq1_4'
        assert result.reference_protein_start == 57
        assert result.reference_protein_stop == 162
        assert result.reference_protein_length == 295

        assert result.input_protein_start == 7
        assert result.input_protein_stop == 118
        assert result.input_protein_length == 140



#!/usr/bin/env python

from collections import OrderedDict
from .Interfaces import hAMRonizedResultIterator
from hAMRonization.constants import GENE_PRESENCE
import re

required_metadata = [
    "analysis_software_version",
    "reference_database_version",
    "input_file_name",
]


class FARGeneIOIterator(hAMRonizedResultIterator):
    def __init__(self, source, metadata):
        metadata["analysis_software_name"] = "fargene"
        metadata["reference_database_name"] = "fargene_hmms"
        metadata["genetic_variation_type"] = GENE_PRESENCE
        self.metadata = metadata

        # needed as indexing into the positions
        self.field_mapping = OrderedDict(
            [
                ("# target name", 'input_sequence_id'),
                ("accession", None),
                ("tlen", 'input_protein_length'),
                ("query name", 'reference_accession'),
                ("accession2", None),
                ("qlen", "reference_protein_length"),
                ("full_E-value", None),
                ("full_score", None),
                ("full_bias", None),
                ("dom_#", None),
                ("dom_of", None),
                ("dom_c-Evalue", None),
                ("dom_i-Evalue", None),
                ("dom_score", None),
                ("dom_bias", None),
                ("hmm_from", 'reference_protein_start'),
                ("hmm_to", 'reference_protein_stop'),
                ("ali_from", 'input_protein_start'),
                ("ali_to", 'input_protein_stop'),
                ("env_from", None),
                ("env_to", None),
                ("acc", None),
                ("description of target", None),
                ("_drug_class", 'drug_class'),
                ("_gene_name", "gene_name"),
                ("_gene_symbol", "gene_symbol"),
            ]
        )

        super().__init__(source, self.field_mapping, self.metadata)

    def parse(self, handle):
        """
        Read each and return it
        """
        # skip any manually specified fields for later
        report_fieldnames = [
            x for x in self.field_mapping.keys() if not x.startswith("_")
        ]

        for result in handle:
            if not result.startswith("#"):
                result = dict(zip(report_fieldnames, re.split(r'\s+', result)))
                result["_gene_name"] = result['query name']
                result['_gene_symbol'] = result['query name'].split('_')[0]
                result['_drug_class'] = result['query name'].split('_')[0]

                yield self.hAMRonize(result, self.metadata)



#!/usr/bin/env python

import csv
from .Interfaces import hAMRonizedResultIterator
from hAMRonization.constants import GENE_PRESENCE

required_metadata = [
    "analysis_software_version",
    "reference_database_version",
    "reference_database_name",
    "input_file_name",
]


class SraxIterator(hAMRonizedResultIterator):
    def __init__(self, source, metadata):
        metadata["analysis_software_name"] = "srax"
        metadata["genetic_variation_type"] = GENE_PRESENCE
        self.metadata = metadata

        self.field_mapping = {
            "Locus ID": None,
            "# Sequences": None,
            "ARG": "gene_symbol",
            "Coverage (%)": "coverage_percentage",
            "Identity (%)": "sequence_identity",
            "Drug class": "drug_class",
            "Gene accession ID": "reference_accession",
            "Gene description": "gene_name",
            "AMR detection model": None,
        }

        super().__init__(source, self.field_mapping, self.metadata)

    def parse(self, handle):
        """
        Read each and return it
        """
        # skip any manually specified fields for later
        reader = csv.DictReader(handle, delimiter="\t")
        for result in reader:
            yield self.hAMRonize(result, self.metadata)



#!/usr/bin/env python

import csv
from .Interfaces import hAMRonizedResultIterator
from hAMRonization.constants import GENE_PRESENCE

required_metadata = [
    "analysis_software_version",
    "reference_database_version",
    "input_file_name",
]


class ResFinderIterator(hAMRonizedResultIterator):
    """
    Updated for ResFinder v4.1 using the `ResFinder_results_tab.txt` output
    file
    """

    def __init__(self, source, metadata):
        metadata["reference_database_name"] = "resfinder"
        metadata["analysis_software_name"] = "resfinder"
        # even though resfinderv4 runs pointfinder
        # parsing mutational resistance requires parsing a different file
        # to get mutations (pointfinder)
        metadata["genetic_variation_type"] = GENE_PRESENCE
        self.metadata = metadata

        self.field_mapping = {
            "Resistance gene": "gene_symbol",
            "Identity": "sequence_identity",
            "Alignment Length/Gene Length": None,
            "Position in reference": None,
            "Contig": "input_sequence_id",
            "Position in contig": None,
            "Accession no.": "reference_accession",
            "Phenotype": "drug_class",
            "Coverage": "coverage_percentage",
            # decomposed from Position in contig e.g "10432..11277"
            "_start": "input_gene_start",
            "_stop": "input_gene_stop",
            # infered from Position in contig field
            "_strand": "strand_orientation",
            # Resistance gene is mapped to both symbol and name
            "_gene_name": "gene_name",
            # decomposed from Alignment Length/Gene Length e.g 1176/1176
            "_reference_gene_length": "reference_gene_length",
        }

        super().__init__(source, self.field_mapping, self.metadata)

    def parse(self, handle):
        """
        Read each and return it
        """
        reader = csv.DictReader(handle, delimiter="\t")
        for result in reader:
            result["_gene_name"] = result["Resistance gene"]
            _start, _stop = result["Position in contig"].split("..")
            if _start > _stop:
                _strand = "-"
            else:
                _strand = "+"
            result["_start"] = _start
            result["_stop"] = _stop
            result["_strand"] = _strand
            _reference_gene_length = result["Alignment Length/Gene Length"].split("/")[
                0
            ]
            result["_reference_gene_length"] = _reference_gene_length
            yield self.hAMRonize(result, self.metadata)
            result = {}



#!/usr/bin/env python

import csv
from .Interfaces import hAMRonizedResultIterator
from hAMRonization.constants import GENE_PRESENCE

required_metadata = [
    "analysis_software_version",
    "reference_database_version",
    "input_file_name",
]


class KmerResistanceIterator(hAMRonizedResultIterator):
    def __init__(self, source, metadata):
        metadata["analysis_software_name"] = "kmerresistance"
        metadata["reference_database_name"] = "resfinder"
        metadata["genetic_variation_type"] = GENE_PRESENCE
        self.metadata = metadata

        self.field_mapping = {
            "#Template": "reference_accession",
            "Score": None,
            "Expected": None,
            "Template_length": "reference_gene_length",
            # should be double checked query/template are right
            "Template_Identity": None,
            "Template_Coverage": "coverage_percentage",
            "Query_Identity": "sequence_identity",
            # should be checked
            "Query_Coverage": None,
            "Depth": "coverage_depth",
            "q_value": None,
            "p_value": None,
            # will be parsed from #Template (only works for resfinder)
            "_gene_name": "gene_name",
            "_gene_symbol": "gene_symbol",
        }

        super().__init__(source, self.field_mapping, self.metadata)

    def parse(self, handle):
        """
        Read each and return it
        """
        # skip any manually specified fields for later
        reader = csv.DictReader(handle, delimiter="\t")
        for result in reader:
            gene_name = "_".join(result["#Template"].split("_")[:-1])
            result["_gene_name"] = gene_name
            result["_gene_symbol"] = result["#Template"].split("_")[0]

            yield self.hAMRonize(result, self.metadata)



#!/usr/bin/env python

import csv
from collections import OrderedDict
from .Interfaces import hAMRonizedResultIterator
from hAMRonization.constants import GENE_PRESENCE

required_metadata = [
    "analysis_software_version",
    "reference_database_version",
    "input_file_name",
    "reference_database_name",
]


class CSStarIterator(hAMRonizedResultIterator):
    def __init__(self, source, metadata):
        metadata["analysis_software_name"] = "csstar"
        metadata["genetic_variation_type"] = GENE_PRESENCE
        self.metadata = metadata
        self.field_mapping = OrderedDict(
            [
                (0, "gene_symbol"),
                (1, "gene_name"),
                (3, "input_sequence_id"),
                (4, "sequence_identity"),
                (5, "input_gene_length"),
                (6, "reference_gene_length"),
                ("_ref", "reference_accession"),
            ]
        )
        super().__init__(source, self.field_mapping, self.metadata)

    def parse(self, handle):
        """
        Read each and return it
        """
        # skip any manually specified fields for later
        field_names = [
            x for x in self.field_mapping.keys() if not str(x).startswith("_")
        ]
        reader = csv.DictReader(handle, fieldnames=field_names, delimiter="\t")
        for result in reader:
            result[0] = (
                result[0]
                .replace("*", "")
                .replace("?", "")
                .replace("TR$", "")
                .replace("$", "")
            )
            result[1] = (
                result[1]
                .replace("*", "")
                .replace("?", "")
                .replace("TR$", "")
                .replace("$", "")
            )
            result["_ref"] = result[1]
            result[4] = result[4].replace("%", "")
            yield self.hAMRonize(result, self.metadata)



#!/usr/bin/env python

import csv
from .Interfaces import hAMRonizedResultIterator
from hAMRonization.constants import NUCLEOTIDE_VARIANT, AMINO_ACID_VARIANT

required_metadata = [
    "analysis_software_version",
    "reference_database_version",
    "input_file_name",
]


class PointFinderIterator(hAMRonizedResultIterator):
    """
    Updated for ResFinder v4.1 using the `PointFinder_results.txt` output
    file
    """

    # Mutation
    # Nucleotide change
    # Amino acid change
    # Resistance
    # PMID

    def __init__(self, source, metadata):
        metadata["reference_database_name"] = "pointfinder"
        metadata["analysis_software_name"] = "pointfinder"
        # even though resfinderv4 runs pointfinder
        # parsing mutational resistance requires parsing a different file
        # to get gene presence absence
        self.metadata = metadata

        self.field_mapping = {
            "Mutation": "reference_accession",
            "Nucleotide change": "nucleotide_mutation",
            "Amino acid change": "amino_acid_mutation",
            "Resistance": "drug_class",
            "PMID": None,
            "_type": "genetic_variation_type",
            "_gene_symbol": "gene_symbol",
            "_gene_name": "gene_name",
        }

        super().__init__(source, self.field_mapping, self.metadata)

    def parse(self, handle):
        """
        Read each and return it
        """
        reader = csv.DictReader(handle, delimiter="\t")
        for result in reader:
            gene, mutation = result["Mutation"].split()
            result["_gene_symbol"] = gene
            result["_gene_name"] = gene

            if mutation.startswith("r."):
                result["_type"] = NUCLEOTIDE_VARIANT
                result["Nucleotide change"] = gene
                result["Amino acid change"] = None
            elif mutation.startswith("p."):
                result["_type"] = AMINO_ACID_VARIANT
                result["Amino acid change"] = mutation
            else:
                raise ValueError(f"Mutation type of {result} not recognised")

            yield self.hAMRonize(result, self.metadata)
            result = {}



#!/usr/bin/env python

import csv
import re
from .Interfaces import hAMRonizedResultIterator
from hAMRonization.constants import (
    NUCLEOTIDE_VARIANT,
    AMINO_ACID_VARIANT,
    GENE_PRESENCE,
)

required_metadata = [
    "analysis_software_version",
    "reference_database_version",
    "input_file_name",
]


class AmrFinderPlusIterator(hAMRonizedResultIterator):
    def __init__(self, source, metadata):
        metadata["analysis_software_name"] = "amrfinderplus"
        metadata["reference_database_name"] = "NCBI Reference Gene Database"
        self.metadata = metadata

        # check source for whether AMFP has been run in protein or nt mode

        nucleotide_field_mapping = {
            "Protein identifier": None,
            "Contig id": "input_sequence_id",
            "Start": "input_gene_start",
            "Stop": "input_gene_stop",
            "Strand": "strand_orientation",
            "Gene symbol": "gene_symbol",
            "Sequence name": "gene_name",
            "Scope": None,
            "Element type": None,
            "Element subtype": None,
            "Class": "drug_class",
            "Subclass": "antimicrobial_agent",
            "Method": None,
            "Target length": "input_protein_length",
            "Reference sequence length": "reference_protein_length",
            "% Coverage of reference sequence": "coverage_percentage",
            "% Identity to reference sequence": "sequence_identity",
            "Alignment length": None,
            "Accession of closest sequence": "reference_accession",
            "Name of closest sequence": None,
            "HMM id": None,
            "HMM description": None,
            "AA Mutation": "amino_acid_mutation",
            "Nucleotide Mutation": "nucleotide_mutation",
            "genetic_variation_type": "genetic_variation_type",
        }
        protein_field_mapping = {
            "Protein identifier": "input_sequence_id",
            "Gene symbol": "gene_symbol",
            "Sequence name": "gene_name",
            "Scope": None,
            "Element": None,
            "Element subtype": None,
            "Class": "drug_class",
            "Subclass": "antimicrobial_agent",
            "Method": None,
            "Target length": "input_protein_length",
            "Reference sequence length": "reference_protein_length",
            "% Coverage of reference sequence": "coverage_percentage",
            "% Identity to reference sequence": "sequence_identity",
            "Alignment length": None,
            "Accession of closest sequence": "reference_accession",
            "Name of closest sequence": None,
            "HMM id": None,
            "HMM description": None,
            "AA Mutation": "amino_acid_mutation",
            "genetic_variation_type": "genetic_variation_type",
        }

        with open(source) as fh:
            _ = next(fh)
            try:
                first_result = next(fh)
                if first_result.strip().split("\t")[0] == "NA":
                    self.field_mapping = nucleotide_field_mapping
                else:
                    self.field_mapping = protein_field_mapping
            except StopIteration:
                # doesn't really matter which mapping as this error indicates
                # this is an empty results file
                self.field_mapping = nucleotide_field_mapping

        super().__init__(source, self.field_mapping, self.metadata)

    def parse(self, handle):
        """
        Read each and return it
        """
        reader = csv.DictReader(handle, delimiter="\t")
        for result in reader:
            # replace NA value with None for consitency
            for field, value in result.items():
                if value == "NA":
                    result[field] = None

            # "POINT" indicates mutational resistance
            # amrfinderplus has no special fields but the mutation itself is
            # appended to the symbol name so we want to split this
            result["AA Mutation"] = None
            result["Nucleotide Mutation"] = None
            result["genetic_variation_type"] = GENE_PRESENCE

            if result["Element subtype"] == "POINT":
                gene_symbol, mutation = result["Gene symbol"].rsplit("_", 1)
                result["Gene symbol"] = gene_symbol
                _, ref, pos, alt, _ = re.split(r"(\D+)(\d+)(\D+)", mutation)
                # this means it is a protein mutation
                if result["Method"] in ["POINTX", "POINTP"]:
                    result["AA Mutation"] = f"p.{ref}{pos}{alt}"
                    result["genetic_variation_type"] = AMINO_ACID_VARIANT
                elif result["Method"] == "POINTN":
                    # e.g., 23S_G2032G ampC_C-11C -> c.2032G>G
                    result["Nucleotide Mutation"] = f"c.{pos}{ref}>{alt}"
                    result["genetic_variation_type"] = NUCLEOTIDE_VARIANT

            yield self.hAMRonize(result, self.metadata)



#!/usr/bin/env python

import os
import dataclasses


@dataclasses.dataclass
class hAMRonizedResult:
    """
    Single AMR result converted to the hAMRonization specification
    Checks types and requires the mandatory fields be supplied
    """

    # mandatory fields
    input_file_name: str
    gene_symbol: str
    gene_name: str
    reference_database_name: str
    reference_database_version: str
    reference_accession: str
    analysis_software_name: str
    analysis_software_version: str
    genetic_variation_type: str

    # optional fields
    antimicrobial_agent: str = None
    coverage_percentage: float = None
    coverage_depth: float = None
    coverage_ratio: float = None
    drug_class: str = None
    input_gene_length: int = None
    input_gene_start: int = None
    input_gene_stop: int = None
    input_protein_length: int = None
    input_protein_start: int = None
    input_protein_stop: int = None
    input_sequence_id: str = None
    nucleotide_mutation: str = None
    nucleotide_mutation_interpretation: str = None
    predicted_phenotype: str = None
    predicted_phenotype_confidence_level: str = None
    amino_acid_mutation: str = None
    amino_acid_mutation_interpretation: str = None
    reference_gene_length: int = None
    reference_gene_start: int = None
    reference_gene_stop: int = None
    reference_protein_length: int = None
    reference_protein_start: int = None
    reference_protein_stop: int = None
    resistance_mechanism: str = None
    strand_orientation: str = None
    sequence_identity: float = None

    def __post_init__(self):
        """
        Use type hints to check if field value is correct value and if not
        try to cast the type (failing with a valueerror)

        Ensure the input_file_name path is just the basename due to different
        tools reporting this differently
        """
        for field in dataclasses.fields(self):
            value = getattr(self, field.name)
            if not isinstance(value, field.type) and value:
                try:
                    setattr(self, field.name, field.type(value))
                except ValueError:
                    raise ValueError(
                        f"Expected {field.name} "
                        f"to be {field.type}, "
                        f"got {repr(value)}"
                    )

        # normalise input filename to just basename without extension
        # this is to ensure compatibility with all tools using the lowest
        # common denominator staramr which does this
        input_file_name = getattr(self, "input_file_name")
        input_file_name = os.path.basename(input_file_name)

        if input_file_name.endswith(".gz"):
            input_file_name = input_file_name.replace(".gz", "")

        for fasta_suffix in [".fna", ".fasta", ".faa", ".fa"]:
            if input_file_name.endswith(fasta_suffix):
                input_file_name = input_file_name.replace(fasta_suffix, "")

        setattr(self, "input_file_name", input_file_name)



#!/usr/bin/env python

import sys
import os
import csv
import json
import argparse
import dataclasses
from abc import ABC, abstractmethod
import hAMRonization
import hAMRonization.summarize
from .hAMRonizedResult import hAMRonizedResult


class hAMRonizedResultIterator(ABC):
    """
    Base class for the parsers for each AMR detection tool

    This should return an appopriate iterator for results from whatever
    AMR tool report is being parsed
    """

    def __init__(self, source, field_map, metadata):
        """
        Create an hAMRonizedResultIterator for whichever tool report is
        being parsed

        Based on:
        github.com/biopython/biopython/blob/master/Bio/SeqIO/Interfaces.py#L23

        Arguments:
            - source: input file stream or path to input
            - tool: name of amr tool report that is being parsed

        """
        self.source = source
        self.field_map = field_map
        self.metadata = metadata

        if os.stat(source).st_size == 0:
            print(f"Warning: {source} is empty", file=sys.stderr)

        try:
            self.stream = open(source, "r")
        except FileNotFoundError:  # path doesn't exist
            print(f"File {source} not found", file=sys.stderr)
            exit(1)

        try:
            self.hAMRonized_results = self.parse(self.stream)
        except Exception:
            self.stream.close()

    def hAMRonize(self, report_data, metadata):
        """
        Convert a line of parsed AMR report in original format to the
        hAMRonization specification
        - report_result parsed dict of single results from report
        - metadata dict of additional metadata fields that need added
        """
        hAMRonized_result_data = {**metadata}

        for original_field, hAMRonized_field in self.field_map.items():
            if hAMRonized_field:
                hAMRonized_result_data[hAMRonized_field] = report_data[original_field]

        hAMRonized_result = hAMRonizedResult(**hAMRonized_result_data)

        return hAMRonized_result

    def __next__(self):
        try:
            return next(self.hAMRonized_results)
        except Exception:
            self.stream.close()
            raise

    def __iter__(self):
        """
        Iterate over entries as an hAMRonizedResult object

        Not to be overwritten in subclasses
        """
        return self

    @abstractmethod
    def parse(self, handle):
        """
        Start parsing the file and return an hAMRonizedResult iterator
        """

    def write(
        self,
        report_number=0,
        total_report_count=1,
        output_location=None,
        output_format="tsv",
    ):
        """
        Class to write to output the hAMRonized report (to either stdout or
        a filehandle) in TSV or json format

        Get number of reports and which report this one is
        """

        if output_location:
            # appending if more than one report
            if os.path.exists(output_location) and total_report_count > 1:
                out_fh = open(output_location, "a")
            else:
                out_fh = open(output_location, "w")
        else:
            out_fh = sys.stdout

        if output_format == "tsv":
            # to get first result to build csvwriter
            try:
                first_result = next(self)
                fieldnames = first_result.__annotations__.keys()
                writer = csv.DictWriter(
                    out_fh,
                    delimiter="\t",
                    fieldnames=fieldnames,
                    lineterminator=os.linesep,
                )

                # donly write header for first report
                if report_number == 0:
                    writer.writeheader()
                writer.writerow(dataclasses.asdict(first_result))
                for result in self:
                    writer.writerow(dataclasses.asdict(result))
            # if the iterator is empty then do nothing
            except StopIteration:
                pass

        # this is painful to do streaming for json validly (requires
        # tinkering with sublcassing the json encoder)
        elif output_format == "json":
            empty = True

            if report_number == 0:
                first_entry = True
            else:
                first_entry = False

            for result in self:
                # replace none entries with empty string for compatibility
                # with csv and non-python
                clean_results = {}
                for key, value in dataclasses.asdict(result).items():
                    if value:
                        clean_results[key] = str(value)
                    else:
                        clean_results[key] = ""

                json_entry = json.dumps(clean_results)

                # add json list opening if first entry
                if first_entry:
                    out_fh.write("[")
                    out_fh.write(json_entry)
                    first_entry = False
                else:
                    out_fh.write(", ")
                    out_fh.write(json_entry)
                empty = False

            if (total_report_count - 1) == report_number:
                # i.e. if last report then close list in json
                if not empty:
                    out_fh.write("]\n")
                else:
                    out_fh.write("[]\n")

        else:
            raise ValueError("Unknown output format. Valid options are: csv or json")

        if out_fh is not sys.stdout:
            out_fh.close()


def generate_tool_subparser(subparser, analysis_tool):
    """
    Build the argument parser for a specific tool
    (used to generate a tool-specific cli-parser and a generic tool parser)
    """
    report_file = hAMRonization._ReportFileToUse[analysis_tool]
    description = (
        f"Applies hAMRonization specification to output(s) from "
        f"{analysis_tool} ({report_file})"
    )
    usage = f"hamronize.py {analysis_tool} <options>"
    help = f"hAMRonize {analysis_tool}'s output report i.e., {report_file}"

    tool_parser = subparser.add_parser(
        analysis_tool, description=description, usage=usage, help=help
    )

    tool_parser.add_argument("report", nargs="+", help="Path to report(s)")
    tool_parser.add_argument(
        "--format", default="tsv", help="Output format (tsv or json)"
    )
    tool_parser.add_argument("--output", default=None, help="Output location")

    # any missing mandatory fields need supplied as CLI argument
    required_mandatory_metadata = hAMRonization._RequiredToolMetadata[analysis_tool]
    for field in required_mandatory_metadata:
        tool_parser.add_argument(
            f"--{field}",
            required=True,
            help=f"Input string containing the {field} " f"for {analysis_tool}",
        )
    return subparser


def generic_cli_interface():
    """
    Generate a generic tool report parser that passes to the tool specific
    parser
    """
    parser = argparse.ArgumentParser(
        description="Convert AMR gene detection "
        "tool output(s) to "
        "hAMRonization specification"
        " format",
        prog="hamronize",
        usage="hamronize <tool> <options>",
    )

    parser.add_argument(
        "-v",
        "--version",
        action="version",
        version=f"%(prog)s {hAMRonization.__version__}",
    )

    # add tool specific parsers
    subparser = parser.add_subparsers(
        title="Tools with hAMRonizable reports", help="", dest="analysis_tool"
    )

    for analysis_tool in hAMRonization._RequiredToolMetadata.keys():
        subparser = generate_tool_subparser(subparser, analysis_tool)

    # add summarize subparser
    # not very pretty to have this tied into the analysis tools list
    # but there still doesn't seem a good way to group subparsers in
    # the argparse library
    description = "Concatenate and summarize AMR detection reports"
    usage = "hamronize summarize <options> <list of reports>"
    summarize_help = "Provide a list of paths to the reports you wish " "to summarize"

    summarize_subparser = subparser.add_parser(
        "summarize", description=description, usage=usage, help=summarize_help
    )

    summarize_subparser.add_argument(
        "-t",
        "--summary_type",
        choices=["tsv", "json", "interactive"],
        default="tsv",
        help="Which summary report format to " "generate",
    )

    summarize_subparser.add_argument(
        "-o", "--output", type=str, default=None, help="Output file path for summary"
    )

    summarize_subparser.add_argument(
        "hamronized_reports", nargs="+", help="list of hAMRonized reports"
    )

    args = parser.parse_args()

    if args.analysis_tool and args.analysis_tool != "summarize":
        required_mandatory_metadata = hAMRonization._RequiredToolMetadata[
            args.analysis_tool
        ]
        metadata = {
            field: getattr(args, field) for field in required_mandatory_metadata
        }

        # parse reports and write as appropriate (only first report with head
        # in tsv mode)
        # check number of reports and append correctly if >1
        total_report_count = len(args.report)
        for report_number, report in enumerate(args.report):
            parsed_report = hAMRonization.parse(report, metadata, args.analysis_tool)

            parsed_report.write(
                report_number=report_number,
                total_report_count=total_report_count,
                output_location=args.output,
                output_format=args.format,
            )

    elif args.analysis_tool == "summarize":
        hAMRonization.summarize.summarize_reports(
            args.hamronized_reports, args.summary_type, args.output
        )
        exit(0)
    else:
        parser.print_help()
        exit(1)



#!/usr/bin/env python

import json
import re
from .Interfaces import hAMRonizedResultIterator
from hAMRonization.constants import NUCLEOTIDE_VARIANT, AMINO_ACID_VARIANT

required_metadata = []


class MykrobeIterator(hAMRonizedResultIterator):
    reference_genomes = {
        "sonnei": "NC_016822.1",  # Shigella sonnei
        "staph": "BX571856.1",  # Staphylococcus aureus
        "tb": "NC_000962.3",  # Mycobacterium tuberculosis,
        "typhi": "AL513382.1",  # Salmonella typhi
    }

    aa_symbols = {
        "A": "Ala",
        "C": "Cys",
        "D": "Asp",
        "E": "Glu",
        "F": "Phe",
        "G": "Gly",
        "H": "His",
        "I": "Ile",
        "K": "Lys",
        "L": "Leu",
        "M": "Met",
        "N": "Asn",
        "P": "Pro",
        "Q": "Gln",
        "R": "Arg",
        "S": "Ser",
        "T": "Thr",
        "V": "Val",
        "W": "Trp",
        "Y": "Tyr",
    }

    def __init__(self, source, metadata):
        metadata["analysis_software_name"] = "Mykrobe"
        self.metadata = metadata

        self.field_mapping = {
            "filename": "input_file_name",
            "gene_symbol": "gene_symbol",
            "gene_name": "gene_name",
            "drug": "drug_class",
            "db_name": "reference_database_name",
            "db_version": "reference_database_version",
            "software_name": "analysis_software_name",
            "mykrobe_version": "analysis_software_version",
            "reference_accession": "reference_accession",
            "nucleotide_mutation": "nucleotide_mutation",
            "amino_acid_mutation": "amino_acid_mutation",
            "nucleotide_mutation_interpretation": "nucleotide_mutation_interpretation",
            "amino_acid_mutation_interpretation": "amino_acid_mutation_interpretation",
            "coverage_percentage": "coverage_percentage",
            "median_coverage_depth": "coverage_depth",
            "type": "genetic_variation_type",
        }

        super().__init__(source, self.field_mapping, self.metadata)

    def parse(self, handle):
        variant_info_re = re.compile(
            r"(?P<gene_symbol>[^_]+)_(?P<aa_change>(?P<aa_from>[A-Z])"
            r"(?P<aa_pos>\d+)(?P<aa_to>[A-Z]))-"
            r"(?P<codon_change>(?P<codon_from>[ACTG]{1,3})"
            r"(?P<codon_pos>\d+)(?P<codon_to>[ACTG]{1,3}))"
        )
        panel_name_re = re.compile(r".*mykrobe/data/(?P<panel>.*)/")
        data = json.load(handle)

        sample_names = list(data.keys())
        assert len(sample_names) == 1, "can only parse output with a single "
        "sample currently, found {}".format(len(sample_names))
        sample_name = sample_names[0]
        panel_name_match = panel_name_re.match(data[sample_name]["probe_sets"][0])
        assert panel_name_match is not None, "can't match panel name "
        "from {}".format(data[sample_name]["probe_sets"][0])
        panel_name = panel_name_match.group("panel")
        if panel_name not in self.reference_genomes:
            raise ValueError("Unknown panel {}".format(panel_name))
        reference_accession = self.reference_genomes[panel_name]
        mykrobe_version = data[sample_name]["version"]["mykrobe-predictor"]
        mykrobe_atlas_version = data[sample_name]["version"]["mykrobe-atlas"]
        db_name = ";".join(
            [
                re.sub(r".*mykrobe/data/(.*)", r"\1", probe_set)
                for probe_set in data[sample_name]["probe_sets"]
            ]
        )

        for drug_name in data[sample_name]["susceptibility"]:
            drug = data[sample_name]["susceptibility"][drug_name]
            if drug["predict"] == "S":
                continue
            for variant in drug["called_by"]:
                variant_match = variant_info_re.match(variant)
                assert (
                    variant_match is not None
                ), "variant_info_re failed to match {}".format(variant)
                gene_symbol = variant_match.group("gene_symbol")
                coverage_percentage = drug["called_by"][variant]["info"]["coverage"][
                    "alternate"
                ]["percent_coverage"]
                median_coverage_depth = drug["called_by"][variant]["info"]["coverage"][
                    "alternate"
                ]["median_depth"]
                ref_median_coverage_depth = drug["called_by"][variant]["info"][
                    "coverage"
                ]["reference"]["median_depth"]
                frequency = median_coverage_depth / (
                    median_coverage_depth + ref_median_coverage_depth
                )

                if len(variant_match.group("codon_from")) == 1:
                    # this not a protein change
                    variant_type = NUCLEOTIDE_VARIANT
                    protein_mutation = (None,)
                else:
                    variant_type = AMINO_ACID_VARIANT
                    protein_mutation = (
                        "p."
                        + self.aa_symbols[variant_match.group("aa_from")]
                        + variant_match.group("aa_pos")
                        + self.aa_symbols[variant_match.group("aa_to")]
                    )
                result = {
                    "filename": handle.name,
                    "gene_symbol": gene_symbol,
                    "gene_name": gene_symbol,
                    "drug": drug_name,
                    "type": variant_type,
                    "software_name": "mykrobe",
                    "mykrobe_version": mykrobe_version,
                    "db_name": db_name,
                    "db_version": mykrobe_atlas_version,
                    "reference_accession": reference_accession,
                    "nucleotide_mutation": variant_match.group(
                        "codon_change"
                    ),  # TODO: make this work using lookup table of gene positions
                    "amino_acid_mutation": protein_mutation,
                    "nucleotide_mutation_interpretation": None,
                    "amino_acid_mutation_interpretation": None,
                    "coverage_percentage": coverage_percentage,
                    "median_coverage_depth": median_coverage_depth,
                    "frequency": frequency,
                }
                yield self.hAMRonize(result, self.metadata)



#!/usr/bin/env python

import csv
from .Interfaces import hAMRonizedResultIterator
from hAMRonization.constants import (
    NUCLEOTIDE_VARIANT,
    AMINO_ACID_VARIANT,
    GENE_PRESENCE,
)

required_metadata = ["analysis_software_version", "reference_database_version"]


class StarAmrIterator(hAMRonizedResultIterator):
    def __init__(self, source, metadata):
        metadata["analysis_software_name"] = "staramr"
        metadata["reference_database_name"] = "resfinder/pointfinder"
        self.metadata = metadata

        self.field_mapping = {
            "Isolate ID": "input_file_name",
            "Gene": "gene_symbol",
            "Predicted Phenotype": "drug_class",
            "%Identity": "sequence_identity",
            "%Overlap": "coverage_percentage",
            "HSP Length/Total Length": "coverage_ratio",
            "Contig": "input_sequence_id",
            "Start": "input_gene_start",
            "End": "input_gene_stop",
            "Accession": "reference_accession",
            "Type": "genetic_variation_type",
            "_nucleotide_mutation": "nucleotide_mutation",
            "_amino_acid_mutation": "amino_acid_mutation",
            # Gene is mapped to both symbol and name
            "Position": None,
            "_gene_name": "gene_name",
        }

        super().__init__(source, self.field_mapping, self.metadata)

    def parse(self, handle):
        """
        Read each and return it
        """
        # skip any manually specified fields for later
        reader = csv.DictReader(handle, delimiter="\t")
        for result in reader:
            coverage_ratio = result["HSP Length/Total Length"].split("/")
            cov_1 = float(coverage_ratio[0])
            cov_2 = float(coverage_ratio[1])
            coverage_ratio = cov_1 / cov_2
            result["HSP Length/Total Length"] = coverage_ratio

            # pointfinder <4
            if "Mutation" in result:
                result["Accession"] = result["Gene"]
                result["_gene_name"] = result["Gene"]

                if result["Type"] == "nucleotide":
                    result["Type"] = NUCLEOTIDE_VARIANT
                    ref, _, alt = result["Mutation"].split()
                    result[
                        "_nucleotide_mutation"
                    ] = f"n.{result['Position']}{ref}>{alt}"
                    result["_amino_acid_mutation"] = None

                elif result["Type"] == "codon":
                    result["Type"] = AMINO_ACID_VARIANT
                    nuc_ref, _, nuc_alt, aa_ref, _, aa_alt = (
                        result["Mutation"].replace("(", "").replace(")", "").split()
                    )
                    result[
                        "_nucleotide_mutation"
                    ] = f"n.{result['Position']}{nuc_ref}>{nuc_alt}"
                    result[
                        "_amino_acid_mutation"
                    ] = f"p.{aa_ref}{result['Position']}{aa_alt}"
                else:
                    assert False

            # resfinder <4
            else:
                result["_gene_name"] = result["Gene"]
                result["Type"] = GENE_PRESENCE
                result["_nucleotide_mutation"] = None
                result["_amino_acid_mutation"] = None

            yield self.hAMRonize(result, self.metadata)



#!/usr/bin/env python

import json
from .Interfaces import hAMRonizedResultIterator
from hAMRonization.constants import NUCLEOTIDE_VARIANT, AMINO_ACID_VARIANT

required_metadata = []


class TBProfilerIterator(hAMRonizedResultIterator):
    def __init__(self, source, metadata):
        metadata["analysis_software_name"] = "tb-profiler"
        self.metadata = metadata

        self.field_mapping = {
            "filename": "input_file_name",
            "gene_symbol": "gene_symbol",
            "gene_name": "gene_name",
            "drug": "drug_class",
            "db_name": "reference_database_name",
            "db_version": "reference_database_version",
            "tbprofiler_version": "analysis_software_version",
            "reference_accession": "reference_accession",
            "nucleotide_mutation": "nucleotide_mutation",
            "amino_acid_mutation": "amino_acid_mutation",
            "nucleotide_mutation_interpretation": "nucleotide_mutation_interpretation",
            "amino_acid_mutation_interpretation": "amino_acid_mutation_interpretation",
            "type": "genetic_variation_type",
        }

        super().__init__(source, self.field_mapping, self.metadata)

    def parse(self, handle):
        """
        Read each and return it
        """
        # skip any manually specified fields for later
        json_obj = json.load(handle)
        for variant in json_obj["dr_variants"]:
            result = {
                "filename": handle.name,
                "gene_symbol": variant["gene"],
                "gene_name": variant["gene"],
                "drug": ";".join([d["drug"] for d in variant["drugs"]]),
                "db_name": json_obj["db_version"]["name"],
                "db_version": json_obj["db_version"]["commit"],
                "type": AMINO_ACID_VARIANT
                if variant["change"][0] == "p"
                else NUCLEOTIDE_VARIANT,
                "tbprofiler_version": json_obj["tbprofiler_version"],
                "reference_accession": variant["feature_id"],
                "nucleotide_mutation": variant["nucleotide_change"],
                "amino_acid_mutation": variant["protein_change"],
                "nucleotide_mutation_interpretation": None,  # These will need to be added in
                "amino_acid_mutation_interpretation": None,  # These will need to be added in
            }
            for drug in variant["drugs"]:
                yield self.hAMRonize(result, self.metadata)



#!/usr/bin/env python

import csv
from .Interfaces import hAMRonizedResultIterator
from hAMRonization.constants import GENE_PRESENCE

required_metadata = ["analysis_software_version", "reference_database_version"]


class AbricateIterator(hAMRonizedResultIterator):
    def __init__(self, source, metadata):
        metadata["analysis_software_name"] = "abricate"
        metadata["genetic_variation_type"] = GENE_PRESENCE
        self.metadata = metadata

        self.field_mapping = {
            "#FILE": "input_file_name",
            "SEQUENCE": "input_sequence_id",
            "START": "input_gene_start",
            "END": "input_gene_stop",
            "STRAND": "strand_orientation",
            "GENE": "gene_symbol",
            "PRODUCT": "gene_name",
            "%COVERAGE": "coverage_percentage",
            "COVERAGE": None,
            "%IDENTITY": "sequence_identity",
            "DATABASE": "reference_database_name",
            "ACCESSION": "reference_accession",
            "RESISTANCE": "drug_class",
            "COVERAGE_MAP": None,
            "GAPS": None,
        }

        super().__init__(source, self.field_mapping, self.metadata)

    def parse(self, handle):
        """
        Read each and return it
        """
        # skip any manually specified fields for later
        reader = csv.DictReader(handle, delimiter="\t")
        for result in reader:
            yield self.hAMRonize(result, self.metadata)



#!/usr/bin/env python

import csv
import pandas as pd
import os
import sys
import json
from .hAMRonizedResult import hAMRonizedResult
import dataclasses
from string import Template


def format_interactive_json(combined_records):
    """
    Reorganised json to row/table format for ease of display
    """
    # dummy field to handle multiple configs
    combined_records["config"] = (
        combined_records["analysis_software_name"].astype(str)
        + combined_records["analysis_software_version"].astype(str)
        + combined_records["reference_database_name"].astype(str)
        + combined_records["reference_database_version"].astype(str)
    )

    configs = combined_records[
        [
            "config",
            "analysis_software_name",
            "analysis_software_version",
            "reference_database_name",
            "reference_database_version",
        ]
    ].drop_duplicates()

    tool_groups = configs.groupby("analysis_software_name")
    configs["display_name"] = configs[
        "analysis_software_name"
    ] + tool_groups.cumcount().apply(lambda x: f": config {x}")

    config_display_names = configs.set_index("config")["display_name"].to_dict()
    combined_records["config_display_name"] = combined_records["config"].apply(
        lambda x: config_display_names[x]
    )

    combined_records = combined_records.drop("config", axis=1)

    data_for_summary = {}

    grouped_data = combined_records.groupby(
        ["input_file_name", "config_display_name"]
    ).apply(lambda x: x.to_json(orient="records"))
    for (input_file, config), hits in grouped_data.items():
        json_hits = json.loads(hits)
        if input_file not in data_for_summary:
            data_for_summary[input_file] = [{config: json_hits}]
        else:
            data_for_summary[input_file].append({config: json_hits})

    tidied_json = []
    for genome in combined_records["input_file_name"].sort_values().unique():
        genome_data = {"input_file_name": genome}
        for config_results in data_for_summary[genome]:
            config = config_results.keys()
            if len(config) != 1:
                raise ValueError
            else:
                config = list(config)[0]
            genome_data[config] = config_results[config]
        tidied_json.append(genome_data)

    tidied_json = json.dumps(tidied_json)
    return tidied_json


def generate_interactive_report(combined_report_data):
    """
    Generate interactive HTML/js report based on what alex sent
    """
    # escape any single quotes
    tidied_json = format_interactive_json(combined_report_data)

    tidied_json = tidied_json.replace("'", "\\'")

    html_template = """<!DOCTYPE html>
    <html>
      <head>
        <title>hAMRonized Results</title>
        <link rel='icon' href="https://pha4ge.org/wp-content/uploads/2020/06/logo-clear.png">
        <!-- Required meta tags -->
        <meta charset="utf-8">
        <meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">
        <!-- Bootstrap CSS -->
        <link rel="stylesheet" href="https://stackpath.bootstrapcdn.com/bootstrap/4.3.1/css/bootstrap.min.css" integrity="sha384-ggOyR0iXCbMQv3Xipma34MD+dH/1fQ784/j6cY/iJTQUOhcWr7x9JvoRxT2MZw1T" crossorigin="anonymous">
        <!-- custom CSS. Must be after botostrap -->
        <!-- pha4ge dark blue: #1c4f77 -->
        <!-- pha4ge light blue: #4879a1 -->
        <style>
          .search_hit{
            background-color: paleturquoise;
          }
          .selected{
            background-color: gold;
          }
          .search_hit.selected{
            background-color: springgreen;
          }
          .amr_hit:hover{
            background-color: lemonchiffon;
          }
          td
            {
              padding:0 15px;
             }
          .phage-blue{
            background-color: #1c4f77;
            color: white;
          }
          .phage-blue-light{
            background-color: #4879a1;
            color: white;
          }
          .sticky{
            position: sticky;
            top: 6rem;
          }
          .selection-bar{
            max-height: 48rem;
          }
          .data-display{
            max-height: 36rem;
          }
          .scroll-x{
            overflow-x: auto;
          }
          .scroll-y{
            overflow-y: auto;
          }
        /* custom table bg-colors
        currently broken. TODO make it work... breaks hover.
        .table-striped>tbody>tr:nth-child(even)>td,
        .table-striped>tbody>tr:nth-child(even)>th {
           background-color: ##99d8c9; // Choose your own color here
         }
       .table-striped>tbody>tr:nth-child(odd)>td,
       .table-striped>tbody>tr:nth-child(odd)>th {
          background-color: #e5f5f9; // Choose your own color here
        } */
        </style>
      </head>
      <body>
        <!-- Navbar -->
        <nav class="navbar sticky-top navbar-light bg-light">
            <a class="navbar-brand" href="#">
              <img src="https://pha4ge.org/wp-content/uploads/2020/04/logob.png" width="320" height="74" alt="">
            </a>
            <div class="form-inline my-2 my-lg-0">
          <input id="gene-search" class="form-control mr-sm-2" onkeyup="geneSearch()" placeholder="Search" aria-label="Search">
          <button id="results-only" class="btn btn-outline-success my-2 my-sm-0" onclick="showOnlyResults()">Show Only Genomes With Hits</button>
          <button id="restore-table" class="btn btn-outline-primary my-2 my-sm-0" onclick="restoreTable()" style="display:none">Restore Results</button>
        </div>

        </nav>
        <div class="container-flex">
          <div class="row">
            <div class="col col-lg-9 col-md-12" id="dynamic-table">
                <!--<input type='button' onclick='CreateTableFromJSON()' value='make table'/>-->
            </div>
            <div class="col">
              <div class="sticky selection-bar scroll-y">
                <div class="row">
                  <div class="col">
                    <div class="card">
                        <div class="card-header phage-blue-light">
                          Search Results
                        </div>
                        <ul class="list-group list-group-flush">
                          <li class="list-group-item d-flex justify-content-between align-items-center">Total hits:<span class="badge badge-primary badge-pill" id="nhits-badge">0</span></li>
                          <li class="list-group-item d-flex justify-content-between align-items-center">Genomes with hits:<span class="badge badge-primary badge-pill" id="ngenomes-badge">0</span></li>
                          <li class="list-group-item d-flex justify-content-between align-items-center">Tools with hits:<span class="badge badge-primary badge-pill" id="ntools-badge">0</span></li>
                          <li class="list-group-item d-flex justify-content-between align-items-center">
                            <div>
                              <span data-toggle="tooltip" data-placement="bottom" class="badge badge-dark text-light badge-pill" title="Indicates the number of genomes in which at least one tool found a hit matching the search query, but not all tools did.">
                                ?
                              </span>
                              Differential results:
                            </div>
                            <span class="badge badge-primary badge-pill" id="ndiff-badge">0</span>
                          </li>
                        </ul>
                      </div>
                    </div>
                  </div>

              <div class="row">
                <div class="col">
                  <div class="card">
                    <div class="card-header phage-blue-light">
                      Selected
                    </div>
                    <ul id="selected-genes" class="list-group list-group-flush">
                      <!--populated by JS -->
                    </ul>
                    <div class="row">
                      <div class="col-6">
                        <button type="button" class="btn btn-info btn-block selection-button" onclick="displayAmrData()" disabled>Compare</button>
                      </div>
                      <div class="col-6">
                        <button type="button" class="btn btn-warning btn-block selection-button" onclick="clearSelected()" disabled>Clear</button>
                      </div>
                    </div>
                  </div>
                </div>
              </div>
            </div>
            </div>
          </div>
       </div>
       <div id="select-container" class="container-flex phage-blue-light">
         <div class="row bg-light">
           <div class="container-fluid"  id="data-display">
           </div>
         </div>
      </div>
     <!-- JavaScript -->
     <!-- jQuery first, then Popper.js, then Bootstrap JS -->
     <script src="https://code.jquery.com/jquery-3.4.1.min.js" integrity="sha256-CSXorXvZcTkaix6Yvo6HppcZGetbYMGWSFlBw8HfCJo=" crossorigin="anonymous"></script>
     <script src="https://cdnjs.cloudflare.com/ajax/libs/popper.js/1.14.7/umd/popper.min.js" integrity="sha384-UO2eT0CpHqdSJQ6hJty5KVphtPhzWj9WO1clHTMGa3JDZwrnQq4sF86dIHNDz0W1" crossorigin="anonymous"></script>
     <script src="https://stackpath.bootstrapcdn.com/bootstrap/4.3.1/js/bootstrap.min.js" integrity="sha384-JjSmVgyd0p3pXB1rRibZUAYoIIy6OrQ6VrjIEaFf/nJGzIxFDsf4x0xIM+B07jRM" crossorigin="anonymous"></script>
     <script type='text/javascript'>
    //parse JSON
    var hamronized_data = JSON.parse('$json_data');



    function CreateTableFromJSON(){
      //get values for html header
      // Tool1, Tool2, Tool3
      var columns = [];
      var toolHits = [];
      for (var i = 0; i < hamronized_data.length; i++){
        for (var toolconfig in hamronized_data[i]){
          if (columns.indexOf(toolconfig) === -1){
            columns.push(toolconfig);

          }
        }
      }

     //Create Table
      var table = document.createElement("table");
      table.setAttribute("class", "table table-striped table-hover");
      table.setAttribute("id", "results-table");
      //make table header row
      var tr = table.insertRow(-1);
      tr.setAttribute("id", "tools-header")
      for (var i = 0; i < columns.length; i++){
        var th = document.createElement("th"); //Header
        // skip putting the input_file_name as the row label header
        if (i == 0) {
            th.innerHTML = '';
        } else {
            th.innerHTML = columns[i];
            //data attribute for search
            th.setAttribute("class", 'tool-column');
            th.setAttribute("data-nhits", '0');
            toolHits.push(0);
        }
        tr.appendChild(th);
      }

      // Add other data as Rows
      for (var i = 0; i < hamronized_data.length; i++){
        //Each Row is a genome. Each Column is a tool.
        //Add some attribute to the rows for interactivity/search
        tr = table.insertRow(-1);
        tr.setAttribute("class", "genome-row");
        tr.setAttribute("data-nhits", '0');
        tr.setAttribute("data-toolhits", toolHits.toString()); //note that JSON is really painfully slow.
    // for each row add the data to the appropriate column
        for (var j=0; j<columns.length; j++){

          var entry = hamronized_data[i][columns[j]];

          //If type is object, we make a list for the entry
          // This is the "list of AMR hits"

          if(typeof entry === 'object' && entry !== null){
            var tableCell = tr.insertCell(-1);
            //record the genome and tool index in each cell. allows better searching
            tableCell.setAttribute("data-genomeIndex", i);
            tableCell.setAttribute("data-toolIndex", j-1); //j-1 bc first column is not a tool.
            //we'll make the list collapsible for better looking table.
            //create a collapse <p> and <button> and add it to the cell
            var numResults = entry.length;
            var dataID = "collapse" + i + j;
            var collapse = document.createElement("p");
            collapse.setAttribute("class", "list-group-item");
            var collapseButton = document.createElement("button");
            collapseButton.setAttribute("class", "btn btn-link");
            collapseButton.setAttribute("type", "button");
            collapseButton.innerHTML = numResults + " hits";
            collapseButton.setAttribute("data-toggle", "collapse");
            collapseButton.setAttribute("data-target", "#"+dataID);
            collapse.appendChild(collapseButton);
            tableCell.appendChild(collapse);

            //create a list
            var list = document.createElement("ul");
            list.setAttribute("class", "list-group collapse");
            list.setAttribute("id", dataID);
            //record number of search hits in the list for display purposes
            list.setAttribute("data-hits", "0");
            //for each amr hit, put the gene_symbol in the list
            for (var k=0; k < entry.length; k++){
              // we can use html5 data-attributes to store data on the cells.
              //var listBullet = document.createElement('li');
              var listBullet = document.createElement("a")
              listBullet.setAttribute("class", "list-group-item list-group-item-action amr_hit");
              //listBullet.setAttribute("href", "#data-display");
              // we can use html5 data-attributes to store data on the cells.
              // The data must be type string.

              var amrData = JSON.stringify(entry[k]);
              listBullet.setAttribute("hamronized_result", amrData);
              listBullet.innerHTML = entry[k].gene_symbol;
              list.appendChild(listBullet);
            }
            tableCell.appendChild(list);
          }

          // if there were no hits
          else if (entry==null){
            var tableCell = tr.insertCell(-1);
            var cell = document.createElement("p");
            cell.setAttribute("class", "list-group-item list-group-item-action disabled");
            cell.innerHTML = "No hits";
            tableCell.appendChild(cell);

          }

          //otherwise, the entry is the input file name
          else{
            //instead of insertCell, manually create the <th> element
            var tableCell = document.createElement("th");
            tableCell.innerHTML = entry;
            //tableCell.setAttribute("class", "input_file_name");
            tableCell.setAttribute("scope", "row");
            tr.appendChild(tableCell);
          }
        }
      }
      //Add the table to a container div
      var divContainer = document.getElementById("dynamic-table");
      divContainer.innerHTML = "";
      divContainer.appendChild(table);

      //Now that the table exists, add OnClick listeners to all the dna-gene class objects
      var geneElements = document.getElementsByClassName('amr_hit');
      for (var i = 0; i < geneElements.length; i++){
        //geneElements[i].addEventListener('click', displayAmrData, false);
        geneElements[i].addEventListener('click', selectGene, false);
      }
    }
    //select or deselect.
    function selectGene(){
      //select
      if (!this.classList.contains('selected')){
        this.classList.add("selected");
      }
      else{
        this.classList.remove("selected")
      }
      //update list of selected genes:
      var allSelected = document.getElementsByClassName("selected");
      var selectedList = document.createElement("ul");
      for (var i=0; i < allSelected.length; i++){
        var listEntry = document.createElement('li');
        listEntry.setAttribute("class", "list-group-item");
        var selectedCell = allSelected[i].parentElement.parentElement;
        var listText = allSelected[i].innerText + " from genome " + selectedCell.dataset.genomeindex + ", tool " + selectedCell.dataset.toolindex;
        listEntry.innerHTML = listText;
        selectedList.appendChild(listEntry);
      }
      var selectedDisplay = document.getElementById('selected-genes');
      var buttons = document.getElementsByClassName('selection-button');
      selectedDisplay.innerHTML = selectedList.innerHTML;
      if (allSelected.length > 0){
        for(var i=0; i<buttons.length; i++){
          buttons[i].removeAttribute("disabled");
        }
      }
      else{
        for(var i=0; i<buttons.length; i++){
          buttons[i].setAttribute("disabled", null);
        }
      }
    }
    //clears the Selected card box.
    function clearSelected(){
      var allSelected = document.getElementsByClassName("selected");
      var cardList = document.getElementById('selected-genes');
      var buttons = document.getElementsByClassName('selection-button');
      while(allSelected.length > 0){
        allSelected[0].classList.remove('selected');
      }
      cardList.innerHTML = "";
      for (var i=0; i<buttons.length; i++){
        buttons[i].setAttribute("disabled", null);
      }
      var tableDiv = document.getElementById('data-display');
      tableDiv.innerHTML = "";
    }

    function displayAmrData(){
      var selectedGenes = document.getElementsByClassName('selected');
      var genomes = document.getElementsByClassName('genome-row');
      var tools = document.getElementsByClassName('tool-column');

      //identify the full set of keys
      function unite() {
          return [].concat.apply([], arguments).filter(function(elem, index, self) {
              return self.indexOf(elem) === index;
          });
      }
       var allKeys = [];
       for (var i=0; i<selectedGenes.length; i++){
         var hamr = JSON.parse(selectedGenes[i].getAttribute("hamronized_result"));
         for (const [key, value] of Object.entries(hamr)){
           allKeys.push(key);
         }
       }
       var uniqueKeys = unite(allKeys);
       //build the table. Columns are selected genes, rows are data keys.
       var table= document.createElement("table");
       table.setAttribute("class", "table table-striped table-hover table-responsive scroll-x scroll-y");
       //make table header row. The row will be comprised of cards indicating the selections
       var tr = table.insertRow(-1);
       //insert a blank header cell for rows
       var thEmpty = document.createElement("th");
       thEmpty.innerHTML="";
       tr.appendChild(thEmpty);
       for (var i=0; i<selectedGenes.length; i++){
         var geneText = selectedGenes[i].innerText;
         var genomeText = genomes[parseInt(selectedGenes[i].parentElement.parentElement.dataset.genomeindex)].firstElementChild.innerText
         var toolText = tools[parseInt(selectedGenes[i].parentElement.parentElement.dataset.toolindex)].innerText

         var th = document.createElement('th');
         // create the header card.
         var card = document.createElement("div");
         card.setAttribute("class","card card-block");
         var cardHeader = document.createElement("div");
         cardHeader.setAttribute("class", "card-header");
         cardHeader.innerText = geneText;
         card.appendChild(cardHeader);
         var cardBody = document.createElement("div");
         cardBody.setAttribute("class","card-body");
         var genome = document.createElement('p');
         var tool = document.createElement('p');
         genome.innerText = genomeText;
         tool.innerText = toolText;
         cardBody.appendChild(genome);
         cardBody.appendChild(tool);
         card.appendChild(cardBody);
         th.appendChild(card);
         tr.appendChild(th);
       }
       //add the other data as rows. Row "headers" will be the uniquekeys
       for (var i=0; i<uniqueKeys.length; i++){
         tr = table.insertRow(-1);
         var tableCell = document.createElement("th");
         tableCell.setAttribute("scope", "row");
         tableCell.innerHTML = uniqueKeys[i];
         tr.appendChild(tableCell);
         for (var j = 0; j<selectedGenes.length; j++){
           amrData = JSON.parse(selectedGenes[j].getAttribute("hamronized_result"));
           //console.log(amrData);
           //console.log(Object.keys(amrData);
           var tableCell = tr.insertCell(-1);
           if (Object.keys(amrData).includes(uniqueKeys[i])){
             tableCell.innerHTML = amrData[uniqueKeys[i]];
           }
           else{
             tableCell.innerHTML = "Not Recorded."
           }
           //tr.appendChild(tableCell)
         }
       }
       var divContainer = document.getElementById('data-display');
       divContainer.innerHTML = "";
       divContainer.appendChild(table);
       //scroll to the result, offset by navbar height
       var scrollTo = document.getElementById('select-container');
       scrollTo.scrollIntoView(true);
       var scrolledY = window.scrollY;
       var navHeight = 85;
       if (scrolledY){
         window.scroll(0, scrolledY - navHeight);
       }

    }

    // Search Function
    // Func will search against the following fields:
    // gene_symbol, gene_name, drug_class, antimicrobial_agent, resistance_mechanism.
    // For all possible fields, see https://github.com/pha4ge/hAMRonization/blob/master/hAMRonization/hAMRonizedResult.py#L23
    function geneSearch() {
     // Declare variables
     var queryTargets =['gene_symbol', 'gene_name', 'drug_class', 'antimicrobial_agent', 'resistance_mechanism'];

     var input = document.getElementById("gene-search");
     var filter = input.value.toUpperCase();
     //var table = document.getElementById("results-table");
     //var cells = table.getElementsByTagName("td");
     var tools = document.getElementsByClassName('tool-column');
     var genomes = document.getElementsByClassName('genome-row');
     // hit stats for floating card
     var hitsDisplay = document.getElementById('nhits-badge');
     var nGenomeDisplay = document.getElementById('ngenomes-badge');
     var nToolsDisplay = document.getElementById('ntools-badge');
     var nDiffDisplay = document.getElementById('ndiff-badge');

     var nHits = parseInt(hitsDisplay.innerText);
     var nGenomes = 0; // parseInt(nGenomeDisplay.innerText);
     var nTools = 0; //parseInt(nToolsDisplay.innerText);
     //console.log(nTools);
     var nDiff = 0; //parseInt(nDiffDisplay.innerText);

     /* Strategy:
     Iterate rows in the table.
     For each row, iterate cells in the row.
     For each cell, check for query matches.
     If a query matches, update : The global hit counter,
                                  The row tool-hit index,
                                  The cell hit counter
     When finished, iterate rows and check each row's tool-hit index to count
     the number of genome hits, tool hits, and differential hits (in which a genome got results from some tools but not all)
     */
     for(var i = 0; i < genomes.length; i++){
       var toolHitIndex = genomes[i].dataset.toolhits.split(',').map(Number); // an array, 0 at index i indicates no hit for tool i
       var cells = genomes[i].getElementsByTagName('td');
       for (var j=0; j<cells.length; j++){
         //check if the cell has any results. If so, check each one for a match
         var results = cells[j].getElementsByTagName("a");
         //console.log(results);
         if (results.length > 0){
           //check if any result matches the query
           for (var k=0; k < results.length; k++){
             //var txtValue = results[k].textContent || results[k].innerText;
             var hamr = JSON.parse(results[k].getAttribute('hamronized_result'));
             var txtValue = "";
             for (const [key, value] of Object.entries(hamr)){
               if (queryTargets.includes(key) && value != "null"){
                 txtValue += value+" ";
               }
             }
             //console.log(txtValue);
             var parentList = results[k].parentElement;

             // if gene name matches query: (TODO check json for full gene name)
             if (txtValue.toUpperCase().indexOf(filter) > -1 && filter != "") {
               if (!results[k].classList.contains('search_hit')){
                 results[k].classList.add("search_hit");
                 //console.log("HIT");
                 //console.log(results[k]);
                 //increment the data-hits attribute for the parent list
                 parentList.setAttribute('data-hits', parseInt(parentList.getAttribute('data-hits'))+1);
                 nHits += 1;
                 genomes[i].dataset.nhits = parseInt(genomes[i].dataset.nhits)+1;
                 //highlight the list header
                 parentList.previousElementSibling.classList.add('search_hit');
                 //expand the list.
                 parentList.classList.add("show");
                 //adjust the tool-hits counter. j is the column index in these loops.
                 toolHitIndex[j] = 1;
               }
             }

             else{
               if (results[k].classList.contains('search_hit')){
                 results[k].classList.remove("search_hit");
                 parentList.setAttribute('data-hits', parseInt(parentList.getAttribute('data-hits'))-1);
                 nHits -= 1;
                 genomes[i].dataset.nhits = parseInt(genomes[i].dataset.nhits)-1;
                 //if there are no hits in the list, un-highlight the list header
                 if (parseInt(parentList.getAttribute('data-hits')) == 0){
                   parentList.previousElementSibling.classList.remove('search_hit');
                   parentList.classList.remove("show");
                   toolHitIndex[j] = 0;

                 }
               }
             }
           }//end amr-hits loop
         }// end results if
       }//end cells loop
       //update genome tool hit index
       genomes[i].setAttribute('data-toolhits', toolHitIndex.toString());
     }//end row loop

     //TODO think of a smarter way to do this. there's too much looping as-is.
     //   Need to look into JS array operations for any efficient shortcuts.
     //loop through genomes and udate the counts
     var globalToolHits = new Array(tools.length).fill(0);
     for (var i=0; i < genomes.length; i++){
       let localToolHits = genomes[i].dataset.toolhits.split(',').map(Number);
       let hitCount = 0;
       for (var j=0; j<localToolHits.length; j++){
         if (localToolHits[j]==1){
           globalToolHits[j]=1;
           //console.log(globalToolHits);
           hitCount++;
         }
       }
       if (hitCount > 0){
         nGenomes++;
         if (hitCount < localToolHits.length){
           nDiff++;
         }
       }
     }
     //console.log(globalToolHits);
     for (var i=0; i < globalToolHits.length; i++){
       if (globalToolHits[i]==1){
         nTools++;
       }
     }
     //update search results card
     hitsDisplay.innerText = nHits;
     nGenomeDisplay.innerText = nGenomes;
     nToolsDisplay.innerText = nTools;
     nDiffDisplay.innerText = nDiff;

    }
    //hides table rows with no results.
    function showOnlyResults(){
     var genomes = document.getElementsByClassName('genome-row');
     var restoreButton = document.getElementById('restore-table');
     for (var i=0; i < genomes.length; i++){
       const counts = genomes[i].dataset.nhits;
       if (counts == 0){
         genomes[i].style.display = "none";
       }
     }
     restoreButton.style.display="";
    }
    //show the full table after hiding no-results
    function restoreTable(){
     var genomes = document.getElementsByClassName('genome-row');
     var restoreButton = document.getElementById('restore-table');
     for (var i=0; i < genomes.length; i++){
       genomes[i].style.display="";
     }
     restoreButton.style.display="none";
    }

        CreateTableFromJSON()
        //initialize tool tips
        $$(function () {
           $$('[data-toggle="tooltip"]').tooltip()
         })


       </script>

     </body>
    </html>
"""  # noqa

    html_template = Template(html_template)

    interactive_report = html_template.substitute(json_data=tidied_json)

    return interactive_report


def check_report_type(file_path):
    """
    Taken from blhsing's answer to stackoverflow.com/questions/54698130
    Identifies whether a report is json or tsv
    """
    with open(file_path) as fh:
        if fh.read(1) in ["{", "["]:
            return "json"
        else:
            fh.seek(0)
            reader = csv.reader(fh, delimiter="\t")
            try:
                if len(next(reader)) == len(next(reader)) > 1:
                    return "tsv"
            except StopIteration:
                return None


def summarize_reports(report_paths, summary_type, output_path=None):
    # fix default output
    if output_path:
        out_fh = open(output_path, "w")
    else:
        out_fh = sys.stdout

    # initialise the combined report with an empty df with just headers
    hamronized_fields = [
            field.name for field in dataclasses.fields(hAMRonizedResult)
        ]
    combined_report_data = [pd.DataFrame(columns=hamronized_fields)]

    report_count = 0

    for report in report_paths:
        if not os.path.exists(report):
            raise FileNotFoundError(f"{report} cannot be found")
        else:
            report_type = check_report_type(report)
            with open(report) as fh:
                # use json library if report is json
                if report_type == "json" or report_type == "interactive":
                    parsed_report = pd.read_json(fh)

                # similarly if the report is a tsv use csv reader
                elif report_type == "tsv":
                    parsed_report = pd.read_csv(fh, sep="\t")

                elif report_type is None:
                    print(f"Warning: {fh} report is empty", file=sys.stderr)
                    parsed_report = pd.DataFrame()
                else:
                    raise FileNotFoundError(
                        f"{report} type cannot be parsed, " "check validity of file"
                    )

        combined_report_data.append(parsed_report)
        report_count += 1

    # remove any duplicate entries in the parsed_report
    # set can't hash dictionaries unfortunately
    combined_reports = pd.concat(combined_report_data, ignore_index=True)
    total_records = len(combined_reports)
    combined_reports = combined_reports.drop_duplicates()

    unique_records = len(combined_reports)
    removed_duplicate_count = total_records - unique_records
    if removed_duplicate_count > 0:
        print(
            f"Warning: {removed_duplicate_count} duplicate records removed",
            file=sys.stderr,
        )


    # sort records by input_file_name, tool_config i.e. toolname, version,
    # db_name, db_versions, and then within that by gene_symbol
    combined_reports = combined_reports.sort_values(
        [
            "input_file_name",
            "analysis_software_name",
            "analysis_software_version",
            "reference_database_name",
            "reference_database_version",
            "gene_symbol",
        ]
    )

    # write the report
    if summary_type == "tsv":
        combined_reports.to_csv(out_fh, sep="\t", index=False)

    elif summary_type == "json":
        combined_reports.to_json(out_fh, orient="records")

    elif summary_type == "interactive":
        interactive_report = generate_interactive_report(combined_reports)
        out_fh.write(interactive_report)

    if output_path:
        print(
            f"Written {report_count} reports with a combined "
            f"{unique_records} unique results to {output_path}",
            file=sys.stderr,
        )
        out_fh.close()



# genetic_variation type associated with detected resistance
NUCLEOTIDE_VARIANT = "nucleotide_variant_detected"
AMINO_ACID_VARIANT = "protein_variant_detected"
GENE_PRESENCE = "gene_presence_detected"



#!/usr/bin/env python

import csv
from collections import OrderedDict
from .Interfaces import hAMRonizedResultIterator
from hAMRonization.constants import GENE_PRESENCE

required_metadata = [
    "analysis_software_version",
    "reference_database_name",
    "reference_database_version",
    "input_file_name",
]


class GrootIterator(hAMRonizedResultIterator):
    def __init__(self, source, metadata):
        metadata["analysis_software_name"] = "groot"
        metadata["genetic_variation_type"] = GENE_PRESENCE
        self.metadata = metadata

        self.field_mapping = OrderedDict(
            [
                ("reference_accession", "reference_accession"),
                ("read_count", "coverage_depth"),
                # depth is getting a bit nebulous
                ("gene_length", "reference_gene_length"),
                ("cigar_string", None),
                ("_gene_name", "gene_name"),
                ("_gene_symbol", "gene_symbol"),
            ]
        )

        super().__init__(source, self.field_mapping, self.metadata)

    def parse(self, handle):
        """
        Read each and return it
        """
        # skip any manually specified fields for later
        field_names = [
            x for x in self.field_mapping.keys() if not str(x).startswith("_")
        ]
        reader = csv.DictReader(handle, delimiter="\t", fieldnames=field_names)
        for result in reader:
            result["_gene_name"] = ".".join(
                result["reference_accession"].split(".")[:3]
            )
            result["_gene_symbol"] = result["reference_accession"].split(".")[0]
            yield self.hAMRonize(result, self.metadata)



#!/usr/bin/env python

import csv
from .Interfaces import hAMRonizedResultIterator
from hAMRonization.constants import GENE_PRESENCE

required_metadata = [
    "analysis_software_version",
    "reference_database_version",
    "input_file_name",
]


class AmrPlusPlusIterator(hAMRonizedResultIterator):
    def __init__(self, source, metadata):
        metadata["analysis_software_name"] = "amrplusplus"
        metadata["reference_database_name"] = "megares"
        metadata["genetic_variation_type"] = GENE_PRESENCE

        self.metadata = metadata
        self.field_mapping = {
            # Sample  Gene    Hits    Gene Fraction
            "Sample": "input_file_name",
            "Gene": None,
            "Gene Fraction": "coverage_percentage",
            # following will be extacted from gene
            "_reference_accession": "reference_accession",
            "_gene_name": "gene_name",
            "_gene_symbol": "gene_symbol",
            "_drug_class": "drug_class",
        }

        super().__init__(source, self.field_mapping, self.metadata)

    def parse(self, handle):
        """
        Read each and return it
        """
        # skip any manually specified fields for later
        reader = csv.DictReader(handle, delimiter="\t")
        for result in reader:
            hit_information = (
                result["Gene"].replace("|RequiresSNPConfirmation", "").split("|")
            )
            result["_reference_accession"] = hit_information[0]
            result["_drug_class"] = hit_information[2]
            result["_gene_symbol"] = hit_information[-1]
            result["_gene_name"] = hit_information[-2]
            yield self.hAMRonize(result, self.metadata)



#!/usr/bin/env python

import csv
import re
from .Interfaces import hAMRonizedResultIterator
from hAMRonization.constants import (
    NUCLEOTIDE_VARIANT,
    AMINO_ACID_VARIANT,
    GENE_PRESENCE,
)

required_metadata = [
    "analysis_software_version",
    "reference_database_version",
    "reference_database_name",
    "input_file_name",
]


class AribaIterator(hAMRonizedResultIterator):
    def __init__(self, source, metadata):
        metadata["analysis_software_name"] = "ariba"
        self.metadata = metadata

        self.field_mapping = {
            # as close as it provides
            "#ariba_ref_name": "reference_accession",
            "ref_name": "gene_name",
            "gene": None,
            "var_only": None,
            "flag": None,
            "reads": None,
            "cluster": "gene_symbol",
            "ref_len": "reference_gene_length",
            "ref_base_assembled": None,
            "pc_ident": "sequence_identity",
            "ctg": "input_sequence_id",
            "ctg_cov": "coverage_depth",
            "known_var": None,
            "var_type": None,
            "var_seq_type": None,
            "known_var_change": None,
            "has_known_var": None,
            "ref_ctg_change": None,
            "ref_ctg_effect": None,
            "ref_start": None,  # for variant
            "ref_end": None,  # for variant
            "ref_nt": None,
            "ctg_start": None,
            "ctg_end": None,
            "smtls_total_depth": None,
            "smtls_nts": None,
            "smtls_nts_depth": None,
            "var_description": None,
            "free_text": None,
            "_gene_symbol": "gene_symbol",
            "_nucleotide_mutation": "nucleotide_mutation",
            "_amino_acid_mutation": "amino_acid_mutation",
            "_genetic_variation_type": "genetic_variation_type",
        }
        super().__init__(source, self.field_mapping, self.metadata)

    def parse(self, handle):
        """
        Read each and return it
        """
        # skip any manually specified fields for later
        reader = csv.DictReader(handle, delimiter="\t")
        for result in reader:
            _gene_symbol = result["ref_name"].split(".")[0]
            result["_gene_symbol"] = _gene_symbol
            # default valuej
            result["_genetic_variation_type"] = GENE_PRESENCE
            result["_nucleotide_mutation"] = None
            result["_amino_acid_mutation"] = None

            if str(result["known_var"]) == "1" and str(result["has_known_var"]) == "1":
                if result["var_seq_type"] == "n":
                    _, ref, pos, alt, _ = re.split(
                        r"(\D+)(\d+)(\D+)", result["known_var_change"]
                    )
                    result["_genetic_variation_type"] = NUCLEOTIDE_VARIANT
                    result["_nucleotide_mutation"] = f"n.{pos}{ref}>{alt}"
                    result["_amino_acid_mutation"] = None
                elif result["var_seq_type"] == "p":
                    result["_genetic_variation_type"] = AMINO_ACID_VARIANT
                    result[
                        "_nucleotide_mutation"
                    ] = f"n.{result['ctg_start']}{result['ref_nt']}>{result['ctg_nt']}"
                    result["_amino_acid_mutation"] = f"p.{result['known_var_change']}"

            yield self.hAMRonize(result, self.metadata)



#!/usr/bin/env python

import csv
from .Interfaces import hAMRonizedResultIterator
from hAMRonization.constants import GENE_PRESENCE

required_metadata = [
    "analysis_software_version",
    "reference_database_version",
    "input_file_name",
]


class Srst2Iterator(hAMRonizedResultIterator):
    def __init__(self, source, metadata):
        metadata["analysis_software_name"] = "srst2"
        metadata["genetic_variation_type"] = GENE_PRESENCE
        self.metadata = metadata

        self.field_mapping = {
            "Sample": "input_file_name",
            "DB": "reference_database_name",
            "gene": "gene_symbol",
            "allele": "gene_name",
            "coverage": "coverage_percentage",
            "depth": "coverage_depth",
            "diffs": None,
            "uncertainty": None,
            "divergence": None,
            # needs checked if this is correct
            "length": "reference_gene_length",
            "maxMAF": None,
            "clusterid": None,
            "seqid": "reference_accession",
            "annotation": None,
        }

        super().__init__(source, self.field_mapping, self.metadata)

    def parse(self, handle):
        """
        Read each and return it
        """
        # skip any manually specified fields for later
        reader = csv.DictReader(handle, delimiter="\t")
        for result in reader:
            yield self.hAMRonize(result, self.metadata)



import setuptools
import re
from distutils.core import setup

with open('README.md') as fh:
    long_description = fh.read()

with open('hAMRonization/__init__.py') as fh:
    info = fh.read()
    version = re.search('^__version__\s*=\s*"(.*)"',
                        info, re.M).group(1)

setup(
    name='hAMRonization',
    packages=['hAMRonization'],
    version=version,
    license="LGPLv3",
    description="Tool to convert and summarize AMR gene detection outputs "
                "using the hAMRonization specification",
    author='Finlay Maguire',
    author_email='finlaymaguire@gmail.com',
    url="https://github.com/pha4ge/hAMRonization",
    download_url=f"https://github.com/pha4ge/archive/v{version}.tar.gz",
    keywords=["Genomics", "Antimicrobial resistance", "Antibiotic",
              "Standardization"],
    python_requires='>=3.7',
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=['pandas'],
    entry_points={
        'console_scripts': [
            'hamronize = hAMRonization.hamronize:main'
            ],
        },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
        "Natural Language :: English",
        "Operating System :: MacOS",
        "Operating System :: POSIX",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Intended Audience :: Science/Research",
        "Intended Audience :: Healthcare Industry",
        "Topic :: Scientific/Engineering :: Bio-Informatics"
        ],
    zip_safe=True,
    )



#!/usr/bin/env python

__version__ = "1.1.1"

from hAMRonization import AbricateIO
from hAMRonization import AmrFinderPlusIO
from hAMRonization import AribaIO
from hAMRonization import RgiIO
from hAMRonization import ResFinderIO
from hAMRonization import SraxIO
from hAMRonization import DeepArgIO
from hAMRonization import KmerResistanceIO
from hAMRonization import Srst2IO
from hAMRonization import GrootIO
from hAMRonization import StarAmrIO
from hAMRonization import CSStarIO
from hAMRonization import AmrPlusPlusIO
from hAMRonization import ResFamsIO
from hAMRonization import TBProfilerIO
from hAMRonization import MykrobeIO
from hAMRonization import PointFinderIO
from hAMRonization import FARGeneIO

_FormatToIterator = {
    "abricate": AbricateIO.AbricateIterator,
    "amrfinderplus": AmrFinderPlusIO.AmrFinderPlusIterator,
    "ariba": AribaIO.AribaIterator,
    "rgi": RgiIO.RgiIterator,
    "resfinder": ResFinderIO.ResFinderIterator,
    "srax": SraxIO.SraxIterator,
    "deeparg": DeepArgIO.DeepArgIterator,
    "kmerresistance": KmerResistanceIO.KmerResistanceIterator,
    "srst2": Srst2IO.Srst2Iterator,
    "groot": GrootIO.GrootIterator,
    "staramr": StarAmrIO.StarAmrIterator,
    "csstar": CSStarIO.CSStarIterator,
    "amrplusplus": AmrPlusPlusIO.AmrPlusPlusIterator,
    "resfams": ResFamsIO.ResFamsIterator,
    "tbprofiler": TBProfilerIO.TBProfilerIterator,
    "mykrobe": MykrobeIO.MykrobeIterator,
    "pointfinder": PointFinderIO.PointFinderIterator,
    "fargene": FARGeneIO.FARGeneIOIterator,
}

_ReportFileToUse = {
    "abricate": "OUTPUT.tsv",
    "amrfinderplus": "OUTPUT.tsv",
    "ariba": "OUTDIR/OUTPUT.tsv",
    "rgi": "OUTPUT.txt or OUTPUT_bwtoutput.gene_mapping_data.txt",
    "resfinder": "ResFinder_results_tab.txt",
    "srax": "sraX_detected_ARGs.tsv",
    "deeparg": "OUTDIR/OUTPUT.mapping.ARG",
    "kmerresistance": "OUTPUT.res",
    "srst2": "OUTPUT_srst2_report.tsv",
    "groot": "OUTPUT.tsv (from `groot report`)",
    "staramr": "resfinder.tsv",
    "csstar": "OUTPUT.tsv",
    "amrplusplus": "gene.tsv",
    "resfams": "resfams.tblout",
    "tbprofiler": "OUTPUT.results.json",
    "mykrobe": "OUTPUT.json",
    "pointfinder": "PointFinder_results.txt",
    "fargene": "retrieved-genes-*-hmmsearched.out"
}


_RequiredToolMetadata = {
    "abricate": AbricateIO.required_metadata,
    "amrfinderplus": AmrFinderPlusIO.required_metadata,
    "amrplusplus": AmrPlusPlusIO.required_metadata,
    "ariba": AribaIO.required_metadata,
    "csstar": CSStarIO.required_metadata,
    "deeparg": DeepArgIO.required_metadata,
    "fargene": FARGeneIO.required_metadata,
    "groot": GrootIO.required_metadata,
    "kmerresistance": KmerResistanceIO.required_metadata,
    "resfams": ResFamsIO.required_metadata,
    "resfinder": ResFinderIO.required_metadata,
    "mykrobe": MykrobeIO.required_metadata,
    "pointfinder": PointFinderIO.required_metadata,
    "rgi": RgiIO.required_metadata,
    "srax": SraxIO.required_metadata,
    "srst2": Srst2IO.required_metadata,
    "staramr": StarAmrIO.required_metadata,
    "tbprofiler": TBProfilerIO.required_metadata,
}


def parse(handle, metadata, tool):
    r"""Turn a sequence file into an iterator returning SeqRecords.
    Arguments:
     - handle   - handle to the file, or the filename as a string
     - tool - lower case string describing the file format.
     - required_arguments - dict containing the required arguments for tool
    Typical usage, opening a file to read in, and looping over the record(s):
    >>> import hAMRonization as hAMR
    >>> filename = "abricate_report.tsv"
    >>> metadata = {"analysis_software_version": "1.0.1",
    ...             "reference_database_version": "2019-Jul-28"}
    >>> for result in hAMR.parse(filename, required_arguments, "abricate"):
    ...    print(result)

    """
    if not isinstance(tool, str):
        raise TypeError("Need a string for the file format (lower case)")
    if not isinstance(metadata, dict):
        raise TypeError("Metadata must be provided as a dictionary")
    if not tool:
        raise ValueError("Tool required (lower case string)")
    if not tool.islower():
        raise ValueError(f"Tool string '{tool}' should be lower case")

    # check all required metadata has been provided
    try:
        tool_required_metadata = _RequiredToolMetadata[tool]
    except KeyError:
        raise ValueError(
            f"Unknown tool: {tool}\nMust be in " f"{_RequiredToolMetadata.keys()}"
        )
    missing_data = []
    for required in tool_required_metadata:
        if required not in metadata:
            missing_data.append(required)
    if missing_data:
        raise ValueError(
            f"{tool} requires {missing_data} supplied " "in metadata dictionary"
        )

    iterator_generator = _FormatToIterator.get(tool)
    if iterator_generator:
        return iterator_generator(handle, metadata)
    raise ValueError(f"Unknown tool: {tool}\nMust be in " f"{_FormatToIterator.keys()}")
